
#include "Header/2dwater.h"

water2d::water2d(){
	int x, z;
	for(z=0;z<WATER_DEPTH;z++){	
		for(x=0;x<WATER_WIDTH;x++){
			a.val[x][z]=	
				b.val[x][z]=0;	
		}
	}
	
	damping = 0.9999999f;
	scalexz	= 4.0f;
	scaley	= 0.03f;
	level	= 2.2f;
	
}

void water2d::Update (){
	int x, z;
	buffer temp;
	temp = a;
	a = b;
	b = temp;
	
	
	for(z=1;z<WATER_DEPTH-1;z++){	
		for(x=1;x<WATER_WIDTH-1;x++){
			
			a.val[x][z] = ((	  
				b.val[x-1][z]+
				b.val[x+1][z]+
				b.val[x][z+1]+
				b.val[x][z-1] ) >> 1) - a.val[x][z];
			
			a.val[x][z] = a.val[x][z] * damping;
			if (a.val[x][z]>200) a.val[x][z]=200;
			if (a.val[x][z]<-2000) a.val[x][z]=-2000;
		}
	}
}


void water2d::AddDrop(int x, int z){
	
	b.val[x][z] = 200;
	
}

void water2d::Draw(float scale){
	
	int x, z;
	float col;
	
	OrthoMode(0, 0, WIN_WIDTH, WIN_HEIGHT);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);			
	//glEnable(GL_BLEND);
	
	glBegin(GL_QUADS);			
	for (z = 0; z<128; z++){
		for (x = 0; x<128; x++){
			col = (a.val[x][z]+100)/200.0f;
			glColor3f(col,col,col);
			glVertex2f(x*scale,			z*scale);
			glVertex2f(x*scale,			z*scale+scale);
			glVertex2f(x*scale+scale,	z*scale+scale);
			glVertex2f(x*scale+scale,	z*scale);
		}
	}
	glEnd();
	
	//glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);	
	glEnable(GL_LIGHTING);	
	PerspectiveMode();	
}


void water2d::smoothout(){
int x, y;
float corner, edge, centre;

	for (y = 1; y<127; y++){
		for (x = 1; x<127; x++){				
				corner =	(	a.val[x-1][y-1]+	a.val[x+1][y-1]+	a.val[x-1][y+1]+	a.val[x+1][y+1] ) / 16.0f;
				edge =		float( a.val[x-1][y]  +	a.val[x+1][y] +		a.val[x][y-1]  +	a.val[x][y+1] ) /  8.0f;
				centre =	a.val[x][y] / 4.0f;				
				smooth.val[x][y] = int(corner + edge + centre);
		}		
	}

//	a = smooth;
}

float water2d::heightAtSmoothPoint(float tx, float ty){
int ix, iy;
float dx, dy;
float m, n, o;

	tx = (0.5f*WATER_WIDTH) + (tx/scalexz);
	ty = (0.5f*WATER_DEPTH) + (ty/scalexz);

	ix = int(tx);
	iy = int(ty);
	dx = tx-ix;
	dy = ty-iy;

	m = smooth.val[ix][iy] + (( smooth.val[ix+1][iy] - smooth.val[ix][iy] ) * dx);
	n = smooth.val[ix][iy+1] + (( smooth.val[ix+1][iy+1] - smooth.val[ix][iy+1] ) * dx);
	o = m + (( n-m ) * (dy));

//printf("\nint (%d, %d)\tdec (%2.2f, %2.2f)\t iheight:  %2.2f, Hx+1:  %2.2f\n m:%2.2f,  n:%2.2f,  o:%2.2f", 
//	   ix, iy, dx, dy, float(elevation.height[ix][iy]),  float(elevation.height[ix+1][iy]), m, n, o);
	//printf("\n---------------------------------------------"); 
return (level*scalexz)+(o*scaley);
}






void water2d::Draw3D(float alpha){
int x, z;
float col;
	glDisable(GL_LIGHTING);
	
	if(alpha<1.0f) glEnable(GL_BLEND);
	
	glPushMatrix();
	glTranslatef(-0.5*scalexz*TERR_WIDTH, 2.2f*scalexz, -0.5*scalexz*TERR_DEPTH);

	glColor4f(1,1,1, alpha);
	for(z=0; z<TERR_DEPTH; z++){
		glBegin(GL_TRIANGLE_STRIP);			
		
		for(x=0; x<TERR_WIDTH-1; x++){
			col = (smooth.val[x][z]+100)/200.0f;

			glColor4f(0.2f,0.2f,col, alpha);			
		//	glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);	
			glVertex3f(x*scalexz, a.val[x][z]*scaley, z*scalexz);
			
			col = (a.val[x][z+1]+100)/200.0f;
			glColor4f(0.2f,0.2f,col, alpha);	
		//	glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
			glVertex3f(x*scalexz, a.val[x][z+1]*scaley, z*scalexz+(scalexz));
		}	
		glEnd();
	}
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);	

}

void water2d::Draw3DWithNormals(float alpha){
int x, z;
float col;
Vec3F vecX, vecZ, normalvect;

	//glDisable(GL_LIGHTING);	
	if(alpha<1.0f) glEnable(GL_BLEND);	
	glPushMatrix();
	glTranslatef(-0.5*scalexz*TERR_WIDTH, 2.2f*scalexz, -0.5*scalexz*TERR_DEPTH);

	glColor4f(1,1,1, alpha);
	for(z=0; z<TERR_DEPTH; z++){
		glBegin(GL_TRIANGLE_STRIP);			
		
		for(x=0; x<TERR_WIDTH-1; x++){

			//***********************
			if ((x > 0) && (x < (TERR_WIDTH-1))) //not an edge vertex
				vecX.set(2, a.val[x+1][z] - a.val[x-1][z], 0);
			else
				vecX.set(1, a.val[x][z], 0);
			
			if ((z > 0) && (z < (TERR_DEPTH-1))) //not an edge vertex
				vecZ.set(0, a.val[x][z+1] - a.val[x][z-1], 2);
			else
				vecZ.set(0, a.val[x][z], 1);

			normalvect = vecX.CrossProduct(vecZ);
			normalvect.Normalize();
			normalvect = normalvect * -1.0f;//adjust normals, shouldnt be necessary
			normal[x][z] = normalvect;
			//***********************

		//	col = (smooth.val[x][z]+100)/200.0f;

			glColor4f(0.2f,0.2f,col, alpha);			
			glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);	
			glVertex3f(x*scalexz, a.val[x][z]*scaley, z*scalexz);
			
			col = (a.val[x][z+1]+100)/200.0f;

			glColor4f(0.2f,0.2f,col, alpha);	
			glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
			glVertex3f(x*scalexz, a.val[x][z+1]*scaley, z*scalexz+(scalexz));
		}	
		glEnd();
	}
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);	

}

void water2d::Draw3DTextured(float alpha, float texscale){

	int x, z;
	float col, tx, tz, Xoffset, Yoffset;
	glDisable(GL_LIGHTING);	
	glEnable(GL_TEXTURE_2D);

	tx = 1.0f/float(TERR_WIDTH);
	tz = 1.0f/float(TERR_DEPTH);

	if(alpha<1.0f) glEnable(GL_BLEND);
	
	glPushMatrix();
	glTranslatef(-0.5*scalexz*TERR_WIDTH, 2.2f*scalexz, -0.5*scalexz*TERR_DEPTH);
	glNormal3f(0,1,0);

	glColor4f(1,1,1, alpha);
	
	for(z=0; z<TERR_DEPTH; z++)
	{
		glBegin(GL_TRIANGLE_STRIP);			
		for(x=0; x<TERR_WIDTH-1; x++)
		{

			Xoffset = a.val[x-1][z] - a.val[x+1][z];
			Yoffset = a.val[x][z-1] - a.val[x][z+1];

			glTexCoord2f((x+(Xoffset*texscale))*texscale, (z+(Yoffset*texscale))*texscale);
			//glColor4f(0.2f,0.2f,col, alpha);			
			glVertex3f(x*scalexz, a.val[x][z]*scaley, z*scalexz);
	//		glVertex3f(x*scalexz, 0, z*scalexz);
			
			col = (a.val[x][z+1]+100)/200.0f;			
			Xoffset = a.val[x-1][(z+1)] - a.val[x+1][(z+1)];
			Yoffset = a.val[x][(z+1)-1] - a.val[x][(z+1)+1];
			glTexCoord2f((x+(Xoffset*texscale))*texscale, ((z+1)+(Yoffset*texscale))*texscale);
			//glColor4f(0.2f,0.2f,col, alpha);		
			glVertex3f(x*scalexz, a.val[x][z+1]*scaley, z*scalexz+(scalexz));
	//		glVertex3f(x*scalexz, 0, z*scalexz+(scalexz));
		}	
		glEnd();
	}
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);	

}

void water2d::Draw3DTexturedSmoothRefracted(float alpha, float texscale, bool under)
{

	int x, z;
	static float a = 0;
	a+=0.08;
	float col, tx, tz, Xoffset, Yoffset, tbvalX, tbvalY;
	glDisable(GL_LIGHTING);	
	glEnable(GL_TEXTURE_2D);

	tx = 1.0f/float(TERR_WIDTH);
	tz = 1.0f/float(TERR_DEPTH);

	if(alpha<1.0f) glEnable(GL_BLEND);
	
	glPushMatrix();
	glTranslatef(-0.5*scalexz*TERR_WIDTH, level*scalexz, -0.5*scalexz*TERR_DEPTH);
	glNormal3f(0,1,0);

	glColor4f(1,1,1, alpha);
	
	if(under == 1)
	{
		glPolygonMode( GL_BACK, GL_FILL );					// Back Face Is Made Of Lines
		glCullFace(GL_FRONT);
		glEnable(GL_CULL_FACE);
	}

	for(z=0; z<TERR_DEPTH; z++)
	{
		glBegin(GL_TRIANGLE_STRIP);			
		for(x=0; x<TERR_WIDTH-1; x++)
		{
			tbvalX = 2*sin(x*z+a);
			tbvalY = 2*cos(x*z+a);
			Xoffset = smooth.val[x-1][z] - smooth.val[x+1][z];
			Yoffset = smooth.val[x][z-1] - smooth.val[x][z+1];

//			glTexCoord2f((x+(Xoffset*texscale))*texscale, (z+(Yoffset*texscale))*texscale);
			glTexCoord2f((x+((tbvalX+Xoffset)*texscale))*texscale, (z+((tbvalY+Yoffset)*texscale))*texscale);
			//glColor4f(0.2f,0.2f,col, alpha);			
			glVertex3f(x*scalexz, smooth.val[x][z]*scaley, z*scalexz);
			
			tbvalX = 2*sin((x*(z+1))+a);
			tbvalY = 2*cos((x*(z+1))+a);
			col = (smooth.val[x][z+1]+100)/200.0f;			
			Xoffset = smooth.val[x-1][(z+1)] - smooth.val[x+1][(z+1)];
			Yoffset = smooth.val[x][(z+1)-1] - smooth.val[x][(z+1)+1];
//			glTexCoord2f((x+(Xoffset*texscale))*texscale, ((z+1)+(Yoffset*texscale))*texscale);
			glTexCoord2f((x+((tbvalX+Xoffset)*texscale))*texscale, ((z+1)+((tbvalY+Yoffset)*texscale))*texscale);
			//glColor4f(0.2f,0.2f,col, alpha);		
			glVertex3f(x*scalexz, smooth.val[x][z+1]*scaley, z*scalexz+(scalexz));
		}	
		glEnd();
	}
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);
	
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	

}

void water2d::Draw3DTexturedSmoothIce(float alpha, float texscale){

	int x, z;
	float tx, tz, Xoffset, Yoffset;
	Vec3F vecX, vecZ, normalvect;

	glDisable(GL_LIGHTING);	
	glEnable(GL_TEXTURE_2D);

	tx = 1.0f/float(TERR_WIDTH);
	tz = 1.0f/float(TERR_DEPTH);

	if(alpha<1.0f) glEnable(GL_BLEND);
	
	glPushMatrix();
	glTranslatef(-0.5*scalexz*TERR_WIDTH, level*scalexz, -0.5*scalexz*TERR_DEPTH);
	glNormal3f(0,1,0);

	glColor4f(1,1,1, alpha);
	
	for(z=0; z<TERR_DEPTH; z++)
	{
		glBegin(GL_TRIANGLE_STRIP);			
		for(x=0; x<TERR_WIDTH-1; x++)
		{

			//***********************
			if ((x > 0) && (x < (TERR_WIDTH-1))) //not an edge vertex
				vecX.set(2, a.val[x+1][z] - a.val[x-1][z], 0);
			else
				vecX.set(1, a.val[x][z], 0);
			
			if ((z > 0) && (z < (TERR_DEPTH-1))) //not an edge vertex
				vecZ.set(0, a.val[x][z+1] - a.val[x][z-1], 2);
			else
				vecZ.set(0, a.val[x][z], 1);

			normalvect = vecX.CrossProduct(vecZ);
			normalvect.Normalize();
			normalvect = normalvect * -1.0f;//adjust normals, shouldnt be necessary
			normal[x][z] = normalvect;
			//***********************


			Xoffset = smooth.val[x-1][z] - smooth.val[x+1][z];
			Yoffset = smooth.val[x][z-1] - smooth.val[x][z+1];

			glTexCoord2f((x+(Xoffset*texscale))*texscale, (z+(Yoffset*texscale))*texscale);
			glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);
			glVertex3f(x*scalexz, smooth.val[x][z]*scaley, z*scalexz);
			
			Xoffset = smooth.val[x-1][(z+1)] - smooth.val[x+1][(z+1)];
			Yoffset = smooth.val[x][(z+1)-1] - smooth.val[x][(z+1)+1];
			glTexCoord2f((x+(Xoffset*texscale))*texscale, ((z+1)+(Yoffset*texscale))*texscale);
			glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);
			glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	

			glVertex3f(x*scalexz, smooth.val[x][z+1]*scaley, z*scalexz+(scalexz));
		}	
		glEnd();
	}
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);	

}

void water2d::Draw3DSmoothRefractedTerrain(float alpha, float texscale, terrain &thisTerrain){

	int x, z;
	float col, tx, tz, Xoffset, Yoffset;
	glDisable(GL_LIGHTING);	
	glEnable(GL_TEXTURE_2D);
	tx = 1.0f/float(TERR_WIDTH);
	tz = 1.0f/float(TERR_DEPTH);

	//multi texture ---------------------------------------------------------------------
		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texturelist[4]);
		
		//texture unit 1
		glActiveTextureARB(GL_TEXTURE1_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texturelist[4]);
		
		
	//	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
	//	glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 2);
	//-----------------------------------------------------------------------------------

	if(alpha<1.0f) glEnable(GL_BLEND);
	
	glPushMatrix();
	glTranslatef(-0.5*scalexz*TERR_WIDTH, 2.2f*scalexz, -0.5*scalexz*TERR_DEPTH);
	glNormal3f(0,1,0);

	glColor4f(1,1,1, alpha);
	
	for(z=0; z<TERR_DEPTH; z++)
	{
		glBegin(GL_TRIANGLE_STRIP);			
		for(x=0; x<TERR_WIDTH-1; x++)
		{

			Xoffset = smooth.val[x-1][z] - smooth.val[x+1][z];
			Yoffset = smooth.val[x][z-1] - smooth.val[x][z+1];

	//		glTexCoord2f((x+(Xoffset*texscale))*texscale, (z+(Yoffset*texscale))*texscale);

			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, (x+(Xoffset*texscale))*texscale, (z+(Yoffset*texscale))*texscale);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB, ((0.5*texscale)+x-(Xoffset*texscale))*texscale, ((0.5*texscale)+z-(Yoffset*texscale))*texscale);
			glVertex3f(x*scalexz, smooth.val[x][z]*scaley, z*scalexz);
			
			col = (smooth.val[x][z+1]+100)/200.0f;			
			Xoffset = smooth.val[x-1][(z+1)] - smooth.val[x+1][(z+1)];
			Yoffset = smooth.val[x][(z+1)-1] - smooth.val[x][(z+1)+1];
	//		glTexCoord2f((x+(Xoffset*texscale))*texscale, ((z+1)+(Yoffset*texscale))*texscale);

			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, (x+(Xoffset*texscale))*texscale, ((z+1)+(Yoffset*texscale))*texscale);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB, ((0.5*texscale)+x-(Xoffset*texscale))*texscale, ((0.5*texscale)+(z+1)-(Yoffset*texscale))*texscale);
			glVertex3f(x*scalexz, smooth.val[x][z+1]*scaley, z*scalexz+(scalexz));
	//		glVertex3f(x*scalexz, 0, z*scalexz+(scalexz));
		}	
		glEnd();
	}
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);	

	glActiveTextureARB(GL_TEXTURE1_ARB);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE0_ARB);		
	glDisable(GL_TEXTURE_2D);	

}