
/*
	Copyright A N Houston, University of Abertay Dundee, September 2002.

	$Id$
*/

#include "glut.h"
#include <stdio.h>
//#include "Header/terrain.h"
#include "Header/vectorclass.h"


/*** COMPILATION DIRECTIVES ***/


/*** CONSTANTS ***/

// Initial window/viewport dimensions (in pixels)
#define WIN_WIDTH	(512)
#define WIN_HEIGHT	(512)

// Initial frustum dimensions (in geometric space)
#define FR_WIDTH	((float)0.125)
#define FR_HEIGHT	((float)0.125)
#define FR_NEAR		((float)0.25)
#define FR_FAR		((float)1000)

// Mouse buttons
#define MB_LEFT		(1<<2)
#define MB_MID		(1<<1)
#define MB_RIGHT_	(1<<0)

// Non-printable keys
#define KEY_ESC (27)

#define NOISE_HEIGHT	(1.0)
#define FAULT_HEIGHT	(4.0)
#define BLOB_RADIUS		(8.0)
#define BLOB_HEIGHT		(2.0)

#define MODE_TRACK	(1<<0)
#define MODE_AUTO	(1<<1)

#define RANDOM		(1)
#define MOUNTAIN	(1)


/*** TYPES ***/


/*** MACROS ***/


/*** GLOBALS ***/

Viewport gVP=
{
	{0, 0},
	{WIN_WIDTH, WIN_HEIGHT}
};

Frustum	gFR=
{
	-0.5*FR_WIDTH, 0.5*FR_WIDTH,
	-0.5*FR_HEIGHT, 0.5*FR_HEIGHT,
	FR_NEAR, FR_FAR
};

Mouse gMouse=
{
	{0,0},
	0
};

//TerrainBlock testTB;
//terrain TheTerrain;
//uint plantList= 0;
uint modes= 0;

/*** PRIVATE PROTOTYPES ***/


/*** FUNCTIONS ***/

void initLight (void)
{
	static float fsAmb[]= {0.25, 0.25, 0.25, 1};
	static float l0Dir[]= {0, 1, 0, 0};

	glEnable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	
	glMaterialf(GL_FRONT, GL_SHININESS, 16.0);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);


	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, (GLfloat*)&fsAmb);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_POSITION, l0Dir);
} // initLight

void help (void)
{
	printf("Controls:-\n"); 
	printf("\tLeft mouse drag - set orientation (theta, phi)\n"); 
	printf("\tRight mouse drag - set depth\n");
	printf("\tB - add Blobs (particle accretion)\n");
	printf("\tF - apply geological Faulting\n");
	printf("\tH - Help (show this message)\n");
	printf("\tN - add white Noise\n");
	printf("\tR - Reset\n");
	printf("\tS - Smooth (apply low pass filter)\n");
	printf("\tESC - Exit\n");
} // help

// Set basic OpenGL states
void init (void)
{
	// Have a look at what we're running on (just curious)
	printf("%s\n", (char*)glGetString(GL_VENDOR)); 
	printf("%s\n", (char*)glGetString(GL_RENDERER)); 
	printf("%s\n", (char*)glGetString(GL_VERSION));
	printf("%s\n", (char*)glGetString(GL_EXTENSIONS));

	// Set miscellaneous modes
	glClearColor(0.7f, 0.8f, 0.9f, 1.0f);	// Black background

	glEnable(GL_DEPTH_TEST);

	initLight();

	//terrReset(&testTB);
	

	help();
} // init

void startFrame (void)
{
	// Clear the colour and depth buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Define the viewport within the window (usually full size)
	glViewport(gVP.origin.x, gVP.origin.y, gVP.size.x, gVP.size.y);

	// Now we set up a perspective projection so that the 3D world becomes
	// visible as a 2D image within the viewport.
	// **Following operations are applied to the projection matrix stack**
	glMatrixMode(GL_PROJECTION);	
	// Load the identity matrix (further transformations will be multiplied with this)
	glLoadIdentity();				
	// Define a perspective view volume - this creates a projection matrix
	// and multiples it onto the top of the projection matrix stack
    glFrustum(gFR.left, gFR.right, gFR.bottom, gFR.top, gFR.zNear, gFR.zFar);	

	// **Following operations are applied to the modelling/viewing matrix stack**
	glMatrixMode(GL_MODELVIEW);
	// Load the identity matrix (further transformations will be multiplied with this)
	glLoadIdentity();
} // startFrame

void endFrame (void)
{
	// Swap frame buffers over (makes what we've drawn become visible on screen)
    glutSwapBuffers();
	// Signal that we want the scene redrawn as soon as possible
    glutPostRedisplay();
} // endFrame

void display (void)
{
static float theta= 0, phi= 20, trackX= 0, trackY= -10, depth= 175;

	startFrame();

	// Modify rotation and translation according to user input 
	if (gMouse.buttons & MB_LEFT)
	{
		theta+= gMouse.drag.x;
		phi-= gMouse.drag.y;

		gMouse.drag.x= 0;
		gMouse.drag.y= 0;
	}

	if (gMouse.buttons & MB_RIGHT)
	{
		if (modes & MODE_TRACK)
		{
			trackX+= gMouse.drag.x;
			trackY+= gMouse.drag.y;
		}
		else
		{
			depth+= gMouse.drag.y;
		}

		gMouse.drag.x= 0;
		gMouse.drag.y= 0;
	}

	// Rotate then translate the world
    glTranslatef(trackX, trackY, -depth);
	glRotatef(theta, 0,1,0);
	glRotatef(phi, 1,0,0);

	glColor3f(0.5, 1, 0.25);
	
	glEnable(GL_LIGHTING);
	glPolygonMode(GL_BACK, GL_LINE);
	//terrDraw(&testTB);


	endFrame();
} // display

void updateMouse (int x, int y)
{
	// Invert Y axis
	y= gVP.size.y - y;

	// Calculate motion vector
	gMouse.drag.x= x - gMouse.pos.x;
	gMouse.drag.y= y - gMouse.pos.y;

	// Store new position
	gMouse.pos.x= x;
	gMouse.pos.y= y;
} // updateMouse

void mouse (int button, int state, int x, int y)
{
	switch (button)
	{
		case GLUT_LEFT_BUTTON :
			if (state == GLUT_DOWN)
			{
				gMouse.buttons|= MB_LEFT;
			}
			else
			{
				gMouse.buttons&= ~MB_LEFT;
			}
			break;

		case GLUT_MIDDLE_BUTTON :
			if (state == GLUT_DOWN)
			{
				gMouse.buttons|= MB_MID;
			}
			else
			{
				gMouse.buttons&= ~MB_MID;
			}
			break;

		case GLUT_RIGHT_BUTTON :
			if (state == GLUT_DOWN)
			{
				gMouse.buttons|= MB_RIGHT;
			}
			else
			{
				gMouse.buttons&= ~MB_RIGHT;
			}
			break;
	}
	updateMouse(x,y);
} // mouse

void motion (int x, int y)
{
	updateMouse(x,y);
} // motion

void passive (int x, int y)
{
	updateMouse(x,y);
} // passive

void keyboard (unsigned char key, int x, int y)
{
	switch (key)
    {
		case 'a' :
		case 'A' :
			break;

		case 'b' :
		case 'B' :
			break;

		case 'f' :
		case 'F' :
			break;

		case 'h' :
		case 'H' :
			help();
			break;

		case 'n' :
		case 'N' :
			break;

		case 'r' :
		case 'R' :
			break;

		case 'p' :
		case 'P' :
			break;

		case 's' :
		case 'S' :
			break;

		case 't' :
		case 'T' :
			break;

		case KEY_ESC :
			exit(0);
    }
	updateMouse(x,y);
} // keyboard

void reshape (int x, int y)
{
	gFR.left=	x * (-0.5 * FR_WIDTH / WIN_WIDTH);
	gFR.right=	x * (0.5 * FR_WIDTH / WIN_WIDTH);

	gFR.bottom=	y * (-0.5 * FR_HEIGHT / WIN_HEIGHT);
	gFR.top=	y * (0.5 * FR_HEIGHT / WIN_HEIGHT);

	// Set viewport to size of window
	gVP.size.x= x;
	gVP.size.y= y;
} // reshape

int main (int argc, char *argv[])
{
	// Set up basic GLUT stuff
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowSize(WIN_WIDTH, WIN_HEIGHT);
	glutInitWindowPosition(16, 16);
	glutCreateWindow(argv[0]);

	// Initialise program data
	init();

    // Inform GLUT of this programs IO callback functions
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutPassiveMotionFunc(passive);
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);

    // Start the glut window processing loop (never exits)
	glutMainLoop();

	return(0);
} // main
