#ifndef _VECTOR_H
#define _VECTOR_H

/*
	Copyright A N Houston, University of Abertay Dundee, July 2002.

	$Id$
*/

#include <float.h>
#include <math.h>
#include "fastRand.h"


#ifdef __cplusplus
extern "C" {
#endif

/*** COMPILATION DIRECTIVES ***/


/*** CONSTANTS ***/

#define PI	((float)3.14159)


/*** TYPES ***/

typedef unsigned int	uint;
typedef unsigned short	ushort;
typedef unsigned char	uchar;

typedef struct
{
	float x, y, z;
} Vector, *VectorPtr;


/*** MACROS ***/

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

#define SQR(x) ((x)*(x))
#define SQRT(x) ((float)sqrt(x))

#define SIN(x) ((float)sin(x))
#define COS(x) ((float)cos(x))


/*** GLOBALS ***/


/*** FUNCTIONS ***/

// Set a vectors components to specified values
extern VectorPtr vecSet (VectorPtr pV, float x, float y, float z);

// Generate a random vector in the range (-x..+x, -y..+y, -z..+z)
extern VectorPtr vecRand (VectorPtr pV, float x, float y, float z);

// Randomly perturb a vector by (-x..+x, -y..+y, -z..+z)
extern VectorPtr vecPerturb (VectorPtr pV, float x, float y, float z);

// Generate a random vector of length l
extern VectorPtr vecRandDir (VectorPtr pV, float l);

// Return length (modulus or magnitude) of a vector
extern float vecLength (VectorPtr pV);

// Return dot product of two vectors: V1.V2
extern float vecDot (VectorPtr pV1, VectorPtr pV2);

// Compute sum of two vectors VR= V1 + V2
// Returns pointer to result
extern VectorPtr vecAdd (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2);

// Compute difference between two vectors VR= V1 - V2
// Returns pointer to result
extern VectorPtr vecSub (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2);

// Scale a vector VR= V1 * s
// Returns pointer to result
extern VectorPtr vecScale (VectorPtr pVR, VectorPtr pV1, float s);

// Normalize a vector (convert to unit magnitude) VR= V1 / |V1|
// Returns pointer to result
extern VectorPtr vecNorm (VectorPtr pVR, VectorPtr pV1);

// Compute vector cross product: VR= V1 x V2
// Generates outward facing normals for clockwise vertices.
// Returns pointer to result
extern VectorPtr vecCross (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2);

// Linearly interpolate between a pair of vectors: VR= V1 + (V2-V1) * s
// Returns pointer to result
extern VectorPtr vecLerp (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2, float s);

// Test function to check routines
extern void vecTest (void);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _VECTOR_H
