#ifndef _VECTORCLASS_H_
#define _VECTORCLASS_H_

#include <float.h>
#include <math.h>
#include "Header/fastrand.h"
#include "Header/defines.h"

/*** MACROS ***/

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

#define SQR(x) ((x)*(x))
#define SQRT(x) ((float)sqrt(x))

#define SIN(x) ((float)sin(x))
#define COS(x) ((float)cos(x))

//-----------------------------------------------------
class Vec3F
{
public:
	float x,y,z;

	Vec3F operator + (Vec3F);
	Vec3F operator += (Vec3F);
	Vec3F operator - (Vec3F);
	Vec3F operator -= (Vec3F);
	Vec3F operator * (Vec3F);
	Vec3F operator * (float);
	Vec3F operator *= (float);
	Vec3F operator / (Vec3F); //vector divide
	Vec3F operator /= (float);
	Vec3F operator / (float); //float divide
	Vec3F operator = (float); //float equals

	void set(float nx, float ny, float nz);
	void randomise(float nx, float ny, float nz);
	void preturb(float nx, float ny, float nz);
	float Length();
	float LengthNoRoot();
	void Scale (float s);
	void Normalize();
	float DotProduct (Vec3F &v2);
	Vec3F CrossProduct (Vec3F &v2);
	Vec3F Lerp (Vec3F &v2, float s);
	void RandDir (float l);
	float SeperationXZ (int nx, int nz);

};

class Col3F
{
public:
	float r, g, b;	
	Col3F operator + (Col3F);
	Col3F operator / (float); //float divide
};
typedef unsigned int uint;

typedef struct
{
	float x,y;
} Vec2I;



typedef struct
{
	float r, g, b, a;
} Col4F;

typedef struct
{
	Vec2I origin;
	Vec2I size;
} Viewport;

typedef struct
{
	float left, right;
	float bottom, top;
	float zNear, zFar;
	float zoom;
} Frustum;

typedef struct
{
	Vec2I	pos;
	Vec2I	drag;
	uint	buttons;
	uint	lastbuttons;
} Mouse;



#endif