//=======================================================================================
// camera.cpp  |  terrain demo  |  ca321 (yr2003)	|	10th Nov 2003
//---------------------------------------------------------------------------------------
//
//	Camera class for openGL, 
//
//	based loosely on ultimategame programming.com tutorials.  well, right now the
//	quaternion code is based exactly on this.
//	as a side, these tutorials are pretty good, although the writer seems to fancy 
//	himself as a bit of a twat.  or programming guru to the kings.  but more like a twat.
//
//=======================================================================================


#include "Header/camera.h"

camera::camera(){
position = 0.0f;
lookat = 0;
velocity = 0.0f;
up = 0;
slide = 0;
height = 4.0f;

}

void camera::Set(){
   gluLookAt(	position.x , position.y , position.z ,
				lookat.x , lookat.y , lookat.z ,
				up.x , up.y , up.z );
}

void camera::SetCamera(float x, float y, float z,
					   float vx, float vy, float vz,
					   float ux, float uy, float uz)
{
	
	position.x = x;
	position.y = y;
	position.z = z;

	lookat.x = vx;
	lookat.y = vy;
	lookat.z = vz;

	up.x = ux;
	up.y = uy;
	up.z = uz;
	
}

void camera::Lookat(float x, float y, float z){

	lookat.x = x;
	lookat.y = y;
	lookat.z = z;

	gluLookAt(	position.x , position.y , position.z ,
				lookat.x , lookat.y , lookat.z ,
				up.x , up.y , up.z );

}

void camera::Update(Vec3F direction, float amount){
	
	velocity = direction;
	position += direction * amount;
	lookat += direction * amount;

}

void camera::Advance(float amount){


	//direction we're facing, where were looking - where we are
	Vec3F VecFacing = lookat - position;

	// Normalize this so we can scale it by amount
	VecFacing.Normalize();

	// use update camera to move us
	Update(VecFacing, amount);

}

void camera::Ascend(float amount){

	Update(up, amount);

}


void camera::Strafe(float amount)
{
   Vec3F VecFacing, VecStrafe;


   // Strafing is just like moving the camera forward and backward.  First we will get the
   // direction we are looking.
	VecFacing = lookat - position;

   // Normalize the direction.
	VecFacing.Normalize();


   // Now if we were to call UpdateCamera() we will be moving the camera foward or backwards.
   // We don't want that here.  We want to strafe.  To do so we have to get the cross product
   // of our direction and Up direction view.  The up was set in SetCamera to be 1 positive
   // y.  That is because anything positive on the y is considered up.  After we get the
   // cross product we can save it to the strafe variables so that can be added to the
   // camera using UpdateCamera().

   // Get the cross product of the direction we are looking and the up direction.
	VecStrafe = VecFacing.CrossProduct(up);

   // Save our strafe (cross product) values in xStrafe, yStrafe, and zStrafe.
	Update(VecStrafe, amount);

}


void camera::Rotate(float AngleDir, float amountx, float amounty, float amountz)
{
	quaternion qRotation, qView, qNewView;

   // Create the rotation quaternion based on the axis we are rotating on.
   qRotation.Rotatef(AngleDir, amountx, amounty, amountz);

   // Create the view quaternion.  This will be the direction of the view and position.
   qView.x = lookat.x - position.x;
   qView.y = lookat.y - position.y;
   qView.z = lookat.z - position.z;
   qView.w = 0;

   // Create the resulting quaternion by multiplying the rotation quat by the view quat
   // then multiplying that by the conjugate of the rotation quat.
   qNewView = ((qRotation * qView) * qRotation.Conjugate());

   // Update the view information by adding the position to the resulting quaternion.
   lookat.x = position.x + qNewView.x;
   lookat.y = position.y + qNewView.y;
   lookat.z = position.z + qNewView.z;
}




void camera::RotateByMouse(Mouse tmouse, float midx, float midy, float scale)
{
	float xDirection = 0.0f;         // Direction angle.
	float yRotation = 0.0f;          // Rotation angle.

	// If the mouseX and mouseY are at the middle of the screen then we can't rotate the view.
	//if((tmouse.pos.x == midx) && (tmouse.pos.y == midy))
    //  return;

	// Next we get the direction of each axis.  We divide by a value to get a smaller value back.
	//xDirection = (float)((midx - tmouse.pos.x)) / scale;		
	//yRotation = (float)((midy - tmouse.pos.y)) / scale;		

	xDirection	= -tmouse.drag.x * scale;		
	yRotation	= tmouse.drag.y * scale;		

	// We use curentRotX to help use keep the camera from rotating too far in either direction.
	currentAngle -= yRotation;
/*
	// Stop the camera from going to high...
	if(currentAngle > 30.0f)
      {
		   currentAngle = 30.0f;
         return;
      }

	// Stop the camera from going to low...
	if(currentAngle < -30.0f)
		{
         currentAngle = -30.0f;
         return;
      }
*/
   // Next we get the axis which is a perpendicular vector of the view direction and up values.
   // We use the cross product of that to get the axis then we normalize it.
   Vec3F VecAxis, VecFacing;
   VecAxis = 0.0f;
   VecFacing = 0.0f;

   // Get the Direction of the view.
   VecFacing = lookat - position;

   // Get the cross product of the direction and the up.
   VecAxis = VecFacing.CrossProduct(up);

   // Normalize it.
	VecAxis.Normalize();	

   // Rotate the camera.
   Rotate(yRotation, VecAxis.x, VecAxis.y, VecAxis.z);
   Rotate(xDirection, 0, 1, 0);
}

void camera::SetHeight(float amount){
	
	height += amount;

}

void camera::draw(float scale, float alpha){
float matspec[]= {1.0f, 1.0f, 1.0f, 1};
float matspecx[]= {0.0f, 0.0f, 0.0f, 0};

	glMaterialfv(GL_FRONT, GL_SPECULAR, matspec);
	glMaterialf(GL_FRONT, GL_SHININESS, 64.0f);

	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
		glLineWidth(10.0f);
		glBegin(GL_LINES);
		glColor4f(0.9f, 0.3f, 0.1f, 0.0f);
		glVertex3f(0,50,0);
		glColor4f(0.9f, 0.3f, 0.1f, 0.9f);
		glVertex3f(0,0,0);
		glEnd();
	glEnable(GL_LIGHTING);
		glLineWidth(1.0f);
	glRotatef(90, 1,0,0);
	glutSolidSphere(scale, 32, 16);
	
//	glDisable(GL_BLEND);

	glPopMatrix();
		glDisable(GL_LIGHTING);
		glLineWidth(3.0f);
		glBegin(GL_LINES);
		glColor4f(0.9f, 0.9f, 0.1f, 0.0f);
		glVertex3f(lookat.x, lookat.y, lookat.z);
		glColor4f(0.9f, 0.9f, 0.1f, 0.9f);
		glVertex3f(position.x, position.y, position.z);
		glEnd();
	glEnable(GL_LIGHTING);

	glLineWidth(1.0f);

//	glPopMatrix();

	glDisable(GL_BLEND);

	glMaterialfv(GL_FRONT, GL_SPECULAR, matspecx);

}
