#include "Header/collision.h"


int collisionSphereSphere(Vec3F &posA, float radiusA, Vec3F &posB, float radiusB){
	
	Vec3F pos;
	float distApart, distallowed;
	
	pos = posA - posB;
	distApart = pos.x*pos.x + pos.y*pos.y + pos.z*pos.z;
	distallowed = radiusA+radiusB;
	
	
	return distApart <= distallowed*distallowed;
}

//ugly
int bbox_collision2d(float ax, float ay, float aw, float ah, float bx, float by, float bw, float bh)
{
	
	if(!(((ax + aw) <= bx) 	|| 
		(ax >= (bx + bw)) 	|| 
		((ay + ah) <= by)  	|| 
		(ay >= (by + bh)) ))	
		return 1;
	
	else 	
		return 0;
	
}

int aabb_collision3d(Vec3F &posA, Vec3F &halfSizeA, Vec3F &posB, Vec3F &halfSizeB)
{
	if(!(
		( posA.x + halfSizeA.x <= posB.x - halfSizeB.x )	||
		( posB.x + halfSizeB.x <= posA.x - halfSizeA.x )	||
		
		( posA.y + halfSizeA.y <= posB.y - halfSizeB.y )	||
		( posB.y + halfSizeB.y <= posA.y - halfSizeA.y )	||
		
		( posA.z + halfSizeA.z <= posB.z - halfSizeB.z )	||
		( posB.z + halfSizeB.z <= posA.z - halfSizeA.z )	))
		return 1;
	
	else
		return 0;
	
}



aabb::aabb()
{
	position = 0.0f;
	halfSize = 0.0f;
}

void aabb::draw()
{
	glColor4f(1.0f, 1.0f, 1.0f, 0.6f);
	
	glEnable(GL_BLEND);
	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glBegin(GL_QUADS);
	{
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-halfSize.x, -halfSize.y,  halfSize.z);	// Point 1 (Front)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( halfSize.x, -halfSize.y,  halfSize.z);	// Point 2 (Front)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( halfSize.x,  halfSize.y,  halfSize.z);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-halfSize.x,  halfSize.y,  halfSize.z);	// Point 4 (Front)
		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-halfSize.x, -halfSize.y, -halfSize.z);	// Point 1 (Back)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-halfSize.x,  halfSize.y, -halfSize.z);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( halfSize.x,  halfSize.y, -halfSize.z);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( halfSize.x, -halfSize.y, -halfSize.z);	// Point 4 (Back)
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-halfSize.x,  halfSize.y, -halfSize.z);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-halfSize.x,  halfSize.y,  halfSize.z);	// Point 2 (Top)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( halfSize.x,  halfSize.y,  halfSize.z);	// Point 3 (Top)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( halfSize.x,  halfSize.y, -halfSize.z);	// Point 4 (Top)
		// Bottom Face
		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-halfSize.x, -halfSize.y, -halfSize.z);	// Point 1 (Bottom)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( halfSize.x, -halfSize.y, -halfSize.z);	// Point 2 (Bottom)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( halfSize.x, -halfSize.y,  halfSize.z);	// Point 3 (Bottom)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-halfSize.x, -halfSize.y,  halfSize.z);	// Point 4 (Bottom)
		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( halfSize.x, -halfSize.y, -halfSize.z);	// Point 1 (Right)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( halfSize.x,  halfSize.y, -halfSize.z);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( halfSize.x,  halfSize.y,  halfSize.z);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( halfSize.x, -halfSize.y,  halfSize.z);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-halfSize.x, -halfSize.y, -halfSize.z);	// Point 1 (Left)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-halfSize.x, -halfSize.y,  halfSize.z);	// Point 2 (Left)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-halfSize.x,  halfSize.y,  halfSize.z);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-halfSize.x,  halfSize.y, -halfSize.z);	// Point 4 (Left)
	}
	glEnd();
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
}

int aabb::aabbCollision(aabb &other)
{
	if(!(
		( position.x + halfSize.x <= other.position.x - other.halfSize.x )	||
		( other.position.x + other.halfSize.x <= position.x - halfSize.x )	||
		
		( position.y + halfSize.y <= other.position.y - other.halfSize.y )	||
		( other.position.y + other.halfSize.y <= position.y - halfSize.y )	||
		
		( position.z + halfSize.z <= other.position.z - other.halfSize.z )	||
		( other.position.z + other.halfSize.z <= position.z - halfSize.z )	))
		return 1;
	
	else
		return 0;

}