#include "Header/enemy.h"

enemy::enemy(){
	reset();
}

void enemy::reset(){
	position.x = position.y = position.z = 0.0f;
	velocity.x = velocity.y = velocity.z = 0.0f;
	rotation.x = rotation.y = rotation.z = 0.0f;
	speed = 1.0f;
	friction = 0.8f;
	radius = 5.0f;
	size = 5.0f;
	energy = 100.0f;
	active = 1;
	colour.r = 1.0f;
	colour.g = 0;
	colour.b = 0;
}


void enemy::setHeight(float y){
	position.y = y;
	velocity.y = 0;
}

void enemy::moveInCircle(float amount, float radius){
	static float circlePos;
	
	circlePos += amount;
	position.x = radius * sin(circlePos);
	position.z = radius * cos(circlePos);
	
}

void enemy::moveTowards(Vec3F &tome, float speed){
	Vec3F tempdir;
	tempdir = tome - position;
	tempdir.Normalize();

	velocity+=tempdir;
	velocity.Normalize();
	
	velocity *= speed;
}

void enemy::moveParalell(Vec3F &tome, float speed){
	Vec3F up, facing, strafe;
	up = 0;
	facing = tome - position;
	facing.Normalize();

	strafe = facing.CrossProduct(up);

	velocity+=strafe;
	velocity.Normalize();
	
	velocity *= speed;
}

void enemy::moveRandom(float speed)
{
	Vec3F newdir;

	newdir.randomise(1,1,1);
	
	velocity+=newdir;
	velocity.Normalize();
	
	velocity *= speed;
}


void enemy::AdjustVelocity(float x, float y, float z){
	velocity.x += x ;
	velocity.y += y ;
	velocity.z += z ;
}

void enemy::Update(environment &localWorld){
	
	velocity += localWorld.gravity;
	velocity *= localWorld.viscosity;
	velocity *= friction;
	position += (velocity * speed);
	
}


void enemy::draw(float scale){
	
	if(active == 1){
		if(energy > 0){
			glPushMatrix();
			
			glTranslatef(position.x, position.y+(scale*size*0.5f), position.z);
			glEnable(GL_BLEND);
			glLineWidth(5.0f);
			glPointSize(5.0f);
			glColor4f(1.0f, 0.0f, 0.0f, 0.99f);

			glBegin(GL_POINTS);
			glVertex3f(0,	scale*size*1.5,				0);
			glVertex3f(0,	scale*size*1.5 + 10.0f,		0);
			glEnd();
			
			glColor4f(1.0f, 1.0f, 1.0f, 0.9f);
			glBegin(GL_LINES);
			glVertex3f(0,	scale*size*1.5,				0);
			glVertex3f(0,	scale*size*1.5+energy*0.1f,	0);
			glEnd();
			
			glPointSize(1.0f);
			glLineWidth(1.0f);
			
			//glColor4f(1.0f, 1.0f, 1.0f, 0.9f);
			//glRotatef(90, 1,0,0);
			//glutWireSphere(radius*scale, 16,8);
			glDisable(GL_BLEND);
			
			glColor4f(colour.r, colour.g, colour.b, 1.0f);
			glutSolidCube(scale*size);
			
			
			glPopMatrix();
		}
		else
		{
			drawExplode(scale);
		}
	}
	
}

void enemy::drawExplode(float scale){
	
	if(energy > -100){
		glPushMatrix();
		
		glTranslatef(position.x, position.y+(scale*size*0.5f), position.z);
		glEnable(GL_BLEND);
		glColor4f(1.0f, 0.0f, 1.0f, 0.9f);
		glRotatef(90, 1,0,0);
		//glutWireSphere(-energy--*scale, 16,8);
		glutWireCube(-energy--*scale);
		
		glDisable(GL_BLEND);
		glPopMatrix();
	}
	else
	{
		active = 0;
	}
	
}

//=========================================================================
//
//=========================================================================


agressorEnemy::agressorEnemy()
{

	
	base.colour.r = 0.0f;
	base.colour.g = 0;
	base.colour.b = 1.0f;

	base.size = 10.0f;
	willIShoot = 10;
}

agressorEnemy::shoot(Vec3F &at)
{
	if(randU32()%willIShoot == 1 )
	gun.fire(base.position, at, 2.0f);

}

agressorEnemy::draw(float scale)
{
	base.draw(scale);
	gun.draw(scale*25.0f);

}
