#include "glut.h"
#include <math.h>
#include "Header/particles.h"
#include "Header/emmiter.h"

particle::particle(){

	position.x	= 0;
	position.y	= 1;
	position.z	= 0;

	velocity.x	= 0;
	velocity.y	= 0;
	velocity.z	= 0;

	mass		= 1.0f;
	
	age			= rand()%100;
	birthage	= 100;
	decayrate	= 0.5f;
	scale		= 0.2f;

	texture		= 1;

	alive		= 0;

	colour.r	= 1.0f;
	colour.g	= 1.0f;
	colour.b	= 1.0f;
	colour.a	= 1.0f;
}

void particle::update(environment iamhere, float groundheight, float groundbounce){
	decay();
	gravitate(iamhere);
	collideGroundPlane(groundheight, groundbounce);
	environmentResist(iamhere);
	updatePosition();
}


void particle::updatePosition(){
	oldpos.x = position.x;
	oldpos.y = position.y;
	oldpos.z = position.z;

	position.x += velocity.x;
	position.y += velocity.y;
	position.z += velocity.z;
}

void particle::gravitate(environment thisworld){
	velocity.x += thisworld.gravity.x * mass;
	velocity.y += thisworld.gravity.y * mass;
	velocity.z += thisworld.gravity.z * mass;	
}

void particle::decay(){
age -= decayrate;

	colour.a = age/birthage;
	if(age < 0)
		alive = 0;
	
}

void particle::collideGroundPlane(GLfloat height, GLfloat bounce){
	if((position.y <= height) && (velocity.y <= 0))
		velocity.y *= (elasticity * -bounce);	
}

void particle::environmentResist(environment thisworld){
	velocity.x *= thisworld.viscosity;
	velocity.y *= thisworld.viscosity;
	velocity.z *= thisworld.viscosity;	
}

void particle::drawFlatQuad(float theta, float phi){
	glPushMatrix();
		glColor4f(colour.r, colour.g, colour.b, colour.a);
		glTranslatef(position.x, position.y, position.z);
		glRotatef(-phi, 1,0,0);
		glRotatef(-theta, 0,1,0);
		glBegin(GL_QUADS);
			glVertex3f(scale, -scale, 0);
			glVertex3f(scale, scale, 0);
			glVertex3f(-scale, scale, 0);
			glVertex3f(-scale, -scale, 0);
		glEnd();
	glPopMatrix();
}

void particle::drawTexturedQuad(float theta, float phi){
	
	glBindTexture(GL_TEXTURE_2D, texture);
	glPushMatrix();
		glColor4f(colour.r, colour.g, colour.b, colour.a);
		glTranslatef(position.x, position.y, position.z);
		glRotatef(-phi, 1,0,0);
		glRotatef(-theta, 0,1,0);
		glBegin(GL_QUADS);
			glTexCoord2f(1.0f,1.0f);	glVertex3f(scale, -scale, 0);
			glTexCoord2f(1.0f,0.0f);	glVertex3f(scale, scale, 0);
			glTexCoord2f(0.0f,0.0f);	glVertex3f(-scale, scale, 0);
			glTexCoord2f(0.0f,1.0f);	glVertex3f(-scale, -scale, 0);
		glEnd();
	glPopMatrix();
}

void particle::drawLine(float length, float theta, float phi){

	glLineWidth(scale);
	glBegin(GL_LINES);
		glVertex3f(position.x, position.y, position.z);
		glVertex3f(oldpos.x*length, oldpos.y*length, oldpos.z*length);
	glEnd();

}




/*

void ParticleDisplace(particle *pushme, projectile *pusher, float closest, float furthest, float numerator ){
	
	float xpush, ypush, zpush, xsq, ysq, zsq;
	
	xpush = (pushme->x - pusher->x);
	ypush = (pushme->y - pusher->y);
	zpush = (pushme->z - pusher->z);
	
	xsq = xpush*xpush;
	ysq = ypush*ypush;
	zsq = zpush*zpush;
	
	if(xsq+ysq+zsq<furthest){
		if (xsq > closest)
			pushme->xvel += (numerator/xpush);
		if (ysq > closest)
			pushme->yvel += (numerator/ypush);
		if (zsq > closest)
			pushme->zvel += (numerator/zpush);
	}
	//printf("xpush: %f\tdist: %f\tttlsq: %f\n",(1.0f/xpush),xpush, ttlsq);
	
}



void ParticleAttract(particle *pullme, attractor *tohere){
	if( ( ((pullme->x-tohere->x)*(pullme->x-tohere->x)) 
		+ ((pullme->y-tohere->y)*(pullme->x-tohere->y)) 
		+ ((pullme->z-tohere->z)*(pullme->z-tohere->z)) )<= 	
		(tohere->radius*tohere->radius)){
		
		//---------------------------------------------- x
		if(tohere->x > pullme->x)
			pullme->xvel += tohere->strength;
		else
			if(tohere->x < pullme->x)
			pullme->xvel -= tohere->strength;


			
		//---------------------------------------------- y	
		if(tohere->y > pullme->y)
			pullme->yvel += tohere->strength;
		else
			if(tohere->y < pullme->y)
			pullme->yvel -= tohere->strength;	
		//---------------------------------------------- z			
		if(tohere->z > pullme->z)
			pullme->zvel += tohere->strength;
		else
			if(tohere->z < pullme->z)
			pullme->zvel -= tohere->strength;
	}

}

void ParticleAttractorDraw(attractor *drawme){
	glPushMatrix();
	glTranslatef(drawme->x, drawme->y, drawme->z);
	glColor4f(0.5f,0.5f,0.5f,0.4f);
	glutSolidSphere(0.5f, 8, 4);
		glColor4f(0.5f,0.5f,0.5f,0.12f);
		glutSolidSphere(drawme->radius, 16, 8);	
		glColor4f(0.5f,0.5f,0.5f,0.5f);
		glutSolidSphere(drawme->strength*drawme->radius, 16, 8);	
		
		glColor4f(0.5f,0.5f,0.5f,0.2f);
		glutWireSphere(drawme->radius, 16, 8);	
	glPopMatrix();
}

*/