
/*
	Copyright A N Houston, University of Abertay Dundee, Sept 2001.

	$Id$

*/

#include "Header/plant.h"


/*** COMPILATION DIRECTIVES ***/


/*** CONSTANTS ***/

#define VARIANCE	(0.05)

/*** TYPES ***/


/*** MACROS ***/

#define RV	(randF32()*VARIANCE/2-VARIANCE/2)


/*** GLOBALS ***/

static Plant p=
{
	{
		{2, 8},
		{3, 5},
		{1, 5},
		{-60, 60}
	},
	{3, 6},
	{0.75, 0.95}
};

/*** FUNCTIONS ***/

uint chooseMinMaxU (MinMaxUPtr pM)
{
	return(pM->min + (pM->max - pM->min) * randF32());
} // chooseMinMaxU

float chooseMinMaxF (MinMaxFPtr pM)
{
	return(pM->min + (pM->max - pM->min) * randF32());
} // chooseMinMaxU

void drawBranch (float length)
{
	glColor3f(0.8, 0.4, 0);
	glBegin(GL_LINES);
	glVertex3f(0,0,0);
	glVertex3f(0,length,0);
	glEnd();
} // drawBranch

void drawLeaf (float size)
{
	float s= size*0.2;

	glColor3f(0.2, 0.8, 0);

	glBegin(GL_TRIANGLES);
	
	glVertex3f(0,0,0);
	glVertex3f(RV-s, RV+s, 0);
	glVertex3f(0, size, 0);

	glVertex3f(0,0,0);
	glVertex3f(0, size, 0);
	glVertex3f(RV+s, RV+s, 0);
	
	glEnd();
} // drawLeaf

void drawFlower (float size)
{
	float s= size*0.5;

	// Petals
	glColor3f(1.0, 0.2, 0);

	glBegin(GL_TRIANGLES);
	
	glVertex3f(0,0,0);
	glVertex3f(RV-s, size, RV-s);
	glVertex3f(RV+s, size, RV-s);

	glVertex3f(0,0,0);
	glVertex3f(RV-s, size, RV+s);
	glVertex3f(RV+s, size, RV+s);

	glVertex3f(0,0,0);
	glVertex3f(RV-s, size, RV-s);
	glVertex3f(RV-s, size, RV+s);

	glVertex3f(0,0,0);
	glVertex3f(RV+s, size, RV-s);
	glVertex3f(RV+s, size, RV+s);
	
	glEnd();

	s*= 0.125;
	size*= 0.75;

	// Stamens
	glColor3f(1.0, 1.0, 0);
	glBegin(GL_LINES);
	glVertex3f(0,0,0);
	glVertex3f(RV+s,size,RV+s);
	glVertex3f(0,0,0);
	glVertex3f(RV+s,size,RV+s);
	glVertex3f(0,0,0);
	glVertex3f(RV+s,size,RV+s);
	glEnd();
} // drawFlower

void setTransform (float length, float phi, float theta)
{
	glTranslatef(0, length, 0);
	glRotatef(phi, 0, 1, 0);
	glRotatef(theta, 0, 0, 1);
} // setTransform

void branch (LimbPtr pL, uint level, float scale)
{
	if (level-- > 0)
	{
		uint branches= chooseMinMaxU(&(pL->branches));

		while (branches-- > 0)
		{
			glPushMatrix();
			drawBranch(chooseMinMaxF(&(pL->limbLength))*scale);

			setTransform(chooseMinMaxF(&(pL->branchLength))*scale, randF32()*360.0, chooseMinMaxF(&(pL->theta)));

			branch(pL, level, scale*scale);
			glPopMatrix();
		}
	}
	else
	{
		glPushMatrix();

		setTransform(0, randF32()*360.0, chooseMinMaxF(&(pL->theta)));

		if (randF32() > 0.9)
		{
			drawFlower(randF32()*0.125+0.125);
		}
		else
		{
			drawLeaf(randF32()*0.75+0.25);
		}

		glPopMatrix();
	}
} // branch

void plantDraw (PlantPtr pPL)
{
	pPL= &p;

	branch(&(pPL->limbs), chooseMinMaxU(&(pPL->levels)), chooseMinMaxF(&(pPL->scale)));
} // plantDraw