#include "Header/player.h"

player::player(){
	reset();
}

void player::reset(){
	position.x = position.y = position.z = 0.0f;
	velocity.x = velocity.y = velocity.z = 0.0f;
	rotation.x = rotation.y = rotation.z = 0.0f;
	speed = 1.0f;
	friction = 0.9f;
	circlePos = 0;
	energy = 100;
}

void player::setHeight(float y){
	position.y = y;
	velocity.y = 0;
}


void player::move(bool keys[], float amount){

	if (keys[MY_LEFT]	== 1) AdjustVelocity(-amount,0,0);
	if (keys[MY_RIGHT]	== 1) AdjustVelocity(+amount,0,0);
	if (keys[MY_UP]		== 1) AdjustVelocity(0,0,-amount);
	if (keys[MY_DOWN]	== 1) AdjustVelocity(0,0,+amount);
}

void player::moveInCircle(bool keys[], float amount, float radius){

	if (keys[MY_LEFT]	== 1) AdjustVelocity(-amount,0,0);
	if (keys[MY_RIGHT]	== 1) AdjustVelocity(+amount,0,0);

	circlePos += velocity.x;

	position.x = radius * sin(circlePos);
	position.z = radius * cos(circlePos);

}


void player::AdjustVelocity(float x, float y, float z){
	velocity.x += x ;
	velocity.y += y ;
	velocity.z += z ;
}

void player::Update(environment &localWorld){

	velocity += localWorld.gravity;
	velocity *= localWorld.viscosity;
	velocity *= friction;
	position += (velocity * speed);

}


void player::draw(float scale){
float matspec[]= {1.0f, 1.0f, 1.0f, 1};
float matspecx[]= {0.0f, 0.0f, 0.0f, 0};

//	glMaterialfv(GL_FRONT, GL_SPECULAR, matspec);
//	glMaterialf(GL_FRONT, GL_SHININESS, 64.0f);

	glPushMatrix();

	glTranslatef(position.x, position.y+scale, position.z);
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
		glLineWidth(10.0f);
		glBegin(GL_LINES);
		glColor4f(0.1f, 0.3f, 0.9f, 0.0f);
		glVertex3f(0,50,0);
		glColor4f(0.1f, 0.3f, 0.9f, 0.9f);
		glVertex3f(0,0,0);
		glEnd();
	glEnable(GL_LIGHTING);
	glLineWidth(1.0f);
	glRotatef(90, 1,0,0);
	glutSolidSphere(scale, 32, 16);
	
	glDisable(GL_BLEND);

	glPopMatrix();

	glMaterialfv(GL_FRONT, GL_SPECULAR, matspecx);
}

