
/*
	Copyright A N Houston, University of Abertay Dundee, Sept 2001.

	$Id$

*/

#include "Header/terr.h"


/*** COMPILATION DIRECTIVES ***/


/*** CONSTANTS ***/


/*** TYPES ***/


/*** MACROS ***/


/*** GLOBALS ***/


/*** FUNCTIONS ***/

Vector *geomComputeNormal (Vector *pN, Vector *pV1, Vector *pV2, Vector *pV3)
{
	Vector e1, e2;

	vecSub(&e1, pV2, pV1);
	vecSub(&e2, pV3, pV2);
	vecCross(pN, &e1, &e2);
	return vecNorm(pN, pN);
} // geomComputeNormal

Plane *geomComputePlane (Plane *pPL, Vector *pV1, Vector *pV2, Vector *pV3)
{
	geomComputeNormal(&(pPL->n), pV1, pV2, pV3);
	pPL->s= -vecDot(&(pPL->n), pV1);
	return(pPL);
} // geomComputePlane

float geomPointPlaneSep (Plane *pPL, Vector *pV)
{
	return( vecDot(&(pPL->n), pV) + pPL->s);
} // geomPointPlaneSep

float geomSpherePlaneSep (Plane *pPL, Vector *pC, float r)
{
	return(geomPointPlaneSep(pPL, pC) - r);
} // geomSpherePlaneSep

void terrRandPoint (VectorPtr pP)
{
	vecSet(pP, randF32()*TERR_WIDTH, 0, randF32()*TERR_DEPTH);
} // terrRandPoint

void terrRandPlane (PlanePtr pP)
{
	Vector v;

	// Generate a random direction in the X-Z plane
	vecSet(&v, randF32()-0.5, 0, randF32()-0.5);
	// use as direction for plane normal 
	vecNorm(&(pP->n), &v);

	// Generate a random position in the X-Z plane
	terrRandPoint(&v);
	// use to generate plane separation from origin
	pP->s= -vecDot(&(pP->n), &v);
} // terrRandPlane

void terrApplyFaultPlane (TerrainBlockPtr pTB, PlanePtr pP, float dh)
{
	Vector v;
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			vecSet(&v, x, pTB->h.s[x][z], z);

			if (geomPointPlaneSep(pP, &v) > 0)
			{
				pTB->h.s[x][z]+= dh;
			}
			else
			{
				pTB->h.s[x][z]-= dh;
			}
		}
	}
} // terrApplyFaultPlane

float terrSepXZ (VectorPtr pV, int x, int z)
{
	float dx, dz;

	dx=	pV->x - x;
	dz=	pV->z - z;

	return SQRT(SQR(dx)+SQR(dz));
} // terrSep

void terrAddBlob (TerrainBlockPtr pTB, float r, float h)
{
	Vector v;
	int	x, z, xMin, xMax, zMin, zMax;
	float s;

	terrRandPoint(&v);

	xMin= v.x - r;
	xMax= v.x + r;
	
	zMin= v.z - r;
	zMax= v.z + r;

	xMin= MAX(xMin, 0);
	xMax= MIN(xMax, TERR_WIDTH);
	
	zMin= MAX(zMin, 0);
	zMax= MIN(zMax, TERR_DEPTH);

	z= zMax;
	while (z-- > zMin)
	{
		x= xMax;
		while (x-- > xMin)
		{
			s= terrSepXZ(&v, x, z);

			if (s < r)
			{
				pTB->h.s[x][z]+= h * COS(0.5 * PI * s / r);
			}
		}
	}
} // terrAddBlob

void terrSet (TerrainBlockPtr pTB, float h)
{
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			pTB->h.s[x][z]= h;
		}
	}
} // terrSet

void terrSetNorm (TerrainBlockPtr pTB)
{
	Vector ex, ez;
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			if ((x > 0) && (x < (TERR_WIDTH-1)))
			{
				vecSet(&ex, 2, pTB->h.s[x+1][z] - pTB->h.s[x-1][z], 0);
			}
			else
			{
				vecSet(&ex, 1, pTB->h.s[x][z], 0);
			}

			if ((z > 0) && (z < (TERR_DEPTH-1)))
			{
				vecSet(&ez, 0, pTB->h.s[x][z+1] - pTB->h.s[x][z-1], 2);
			}
			else
			{
				vecSet(&ez, 0, pTB->h.s[x][z], 1);
			}

			vecCross(&(pTB->n[x][z]), &ez, &ex);
			vecNorm(&(pTB->n[x][z]), &(pTB->n[x][z]));
		}
	}
} // terrSetNorm

void terrSmooth (TerrainBlockPtr pTB)
{
	HeightField	newHeight;
	float sum;
	int x, z;

	// ProcesS interior samples
	z= TERR_DEPTH-1;
	while (z-- > 1)
	{
		x= TERR_WIDTH-1;
		while (x-- > 1)
		{
			sum=	pTB->h.s[x-1][z-1]	+	pTB->h.s[x-1][z]	+	pTB->h.s[x-1][z+1] + 
					pTB->h.s[x][z-1]	+	pTB->h.s[x][z]		+	pTB->h.s[x][z+1]   + 
					pTB->h.s[x+1][z-1]	+	pTB->h.s[x+1][z]	+	pTB->h.s[x+1][z+1];

			newHeight.s[x][z]= sum * 1.0 / 9;
		}
	}

	// Process left and right edges
	z= TERR_DEPTH-1;
	while (z-- > 1)
	{
		sum=	pTB->h.s[0][z-1]	+	pTB->h.s[0][z]	+	pTB->h.s[0][z+1]   + 
				pTB->h.s[1][z-1]	+	pTB->h.s[1][z]	+	pTB->h.s[1][z+1];

		newHeight.s[0][z]= sum * 1.0 / 6;

		sum=	pTB->h.s[TERR_WIDTH-1][z-1]	+	pTB->h.s[TERR_WIDTH-1][z]	+	pTB->h.s[TERR_WIDTH-1][z+1]   + 
				pTB->h.s[TERR_WIDTH-2][z-1]	+	pTB->h.s[TERR_WIDTH-2][z]	+	pTB->h.s[TERR_WIDTH-2][z+1];

		newHeight.s[TERR_WIDTH-1][z]= sum * 1.0 / 6;
	}

	// Process far and near edges
	x= TERR_WIDTH-1;
	while (x-- > 1)
	{
		sum=	pTB->h.s[x-1][0]	+	pTB->h.s[x][0]	+	pTB->h.s[x+1][0]   + 
				pTB->h.s[x-1][1]	+	pTB->h.s[x][1]	+	pTB->h.s[x+1][1];

		newHeight.s[x][0]= sum * 1.0 / 6;

		sum=	pTB->h.s[x-1][TERR_DEPTH-1]	+	pTB->h.s[x][TERR_DEPTH-1]	+	pTB->h.s[x+1][TERR_DEPTH-1]   +
				pTB->h.s[x-1][TERR_DEPTH-2]	+	pTB->h.s[x][TERR_DEPTH-2]	+	pTB->h.s[x+1][TERR_DEPTH-2];

		newHeight.s[x][TERR_DEPTH-1]= sum * 1.0 / 6;
	}

	// Process corners
	sum=	pTB->h.s[0][0]	+	pTB->h.s[1][0] +
			pTB->h.s[0][1]	+	pTB->h.s[1][1];

	newHeight.s[0][0]= sum * 1.0 / 4;

	sum=	pTB->h.s[TERR_WIDTH-1][0]	+	pTB->h.s[TERR_WIDTH-2][0] +
			pTB->h.s[TERR_WIDTH-1][1]	+	pTB->h.s[TERR_WIDTH-2][1];

	newHeight.s[TERR_WIDTH-1][0]= sum * 1.0 / 4;

	sum=	pTB->h.s[TERR_WIDTH-1][TERR_DEPTH-1]	+	pTB->h.s[TERR_WIDTH-2][TERR_DEPTH-1] +
			pTB->h.s[TERR_WIDTH-1][TERR_DEPTH-2]	+	pTB->h.s[TERR_WIDTH-2][TERR_DEPTH-2];

	newHeight.s[TERR_WIDTH-1][TERR_DEPTH-1]= sum * 1.0 / 4;

	sum=	pTB->h.s[0][TERR_DEPTH-1]	+	pTB->h.s[0][TERR_DEPTH-1] +
			pTB->h.s[1][TERR_DEPTH-2]	+	pTB->h.s[1][TERR_DEPTH-2];

	newHeight.s[0][TERR_DEPTH-1]= sum * 1.0 / 4;


	// Store back to original
	pTB->h= newHeight;

	terrSetNorm(pTB);
} // terrSmooth

void terrAddNoise (TerrainBlockPtr pTB, float m)
{
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			pTB->h.s[x][z]+= (randF32()-0.5)*m;
		}
	}
	terrSetNorm(pTB);
} // terrAddNoise

void terrGenFaults (TerrainBlockPtr pTB, float dh, uint n)
{
	Plane faultPlane;

	while (n-- > 0)
	{
		terrRandPlane(&faultPlane);
		terrApplyFaultPlane(pTB, &faultPlane, dh);
	}
	terrSetNorm(pTB);
} // terrGenFaults

void terrAddBlobs (TerrainBlockPtr pTB, float r, float h, uint n)
{
	while (n-- > 0)
	{
		terrAddBlob(pTB, r, h);
	}
	terrSetNorm(pTB);
} // terrAddBlobs

void terrReset (TerrainBlockPtr pTB)
{
	//terrRand(pTB);

	terrSet(pTB, 0);

	terrSetNorm(pTB);
} // terrInit

void terrDraw (TerrainBlockPtr pTB)
{
	int x, z;
	int c;

	glPushMatrix();
	glTranslatef(-0.5*TERR_WIDTH, 0, -0.5*TERR_DEPTH);
	glBegin(GL_QUADS);

	z= TERR_DEPTH-1;
	while (z-- > 0)
	{
		x= TERR_WIDTH-1;
		while (x-- > 0)
		{
			c = pTB->h.s[x][z]*.03f;
			glNormal3fv((GLfloat*)&(pTB->n[x][z]));
			glVertex3f(x, pTB->h.s[x][z], z);			
			
			c = pTB->h.s[x][z+1];
			glNormal3fv((GLfloat*)&(pTB->n[x][z+1]));
			glVertex3f(x, pTB->h.s[x][z+1], z+1);
			
			c = pTB->h.s[x+1][z+1];
			glNormal3fv((GLfloat*)&(pTB->n[x+1][z+1]));
			glVertex3f(x+1, pTB->h.s[x+1][z+1], z+1);
			
			c = pTB->h.s[x+1][z];
			glNormal3fv((GLfloat*)&(pTB->n[x+1][z]));
			glVertex3f(x+1, pTB->h.s[x+1][z], z);
		}
	}
	
	glEnd();
	glPopMatrix();
} // terrDraw

