
#include "Header/terrain.h"

terrain::terrain(){

scale = 4.0f;
texscale = 0.1f;
thinking = FALSE;
}
void terrain::Reset()
{
	Set(0);
	CalcNormals();
}

void terrain::Set (float h)
{
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			PlannedElevation.height[x][z]= h;
		}
	}
} // terrSet

void terrain::Update(float h)
{
	int x, z, count;
	
	if(	thinking == TRUE)
	{
		count = 0;	
		z= TERR_DEPTH;
		while (z-- > 0)
		{
			x= TERR_WIDTH;
			while (x-- > 0)
			{
				if ( elevation.height [x][z] < (PlannedElevation.height[x][z]-h) ) 
				{ 
					elevation.height [x][z] += (PlannedElevation.height[x][z] - elevation.height [x][z])*h;
				}
				else if ( elevation.height [x][z] > (PlannedElevation.height[x][z]+h) ) {
					elevation.height [x][z] -= (elevation.height [x][z] - PlannedElevation.height[x][z])*h;
				}
				else
				{ 
					elevation.height [x][z] = PlannedElevation.height[x][z];
					count++;
				}
			}
		}
		
		if(count != TERR_DEPTH*TERR_WIDTH)
		{
			CalcNormals();
		}
		else
		{
			GenDisplayList();
			thinking = FALSE;
		}
		
	}
}

void terrain::UpdateWipe (float h)
{
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			if ( elevation.height [x][z] < (PlannedElevation.height[x][z]-h) ) { elevation.height [x][z] += h; break; }
			if ( elevation.height [x][z] > (PlannedElevation.height[x][z]+h) ) { elevation.height [x][z] -= h; break; }
			
			elevation.height [x][z] = PlannedElevation.height[x][z];

		}
	}

	CalcNormals();

}


void terrain::randomPoint(Vec3F &temp){
//get a random point on the terrain

	temp.set(randF32()*TERR_WIDTH, 0, randF32()*TERR_DEPTH);
	
}



void terrain::AddNoise (float m)
{
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			PlannedElevation.height[x][z]+= (randF32()-0.5)*m;
		}
	}

	CalcNormals();
	thinking = TRUE;
}

void terrain::Draw (int mode, float arb)
{
static float matspec[]= {1.0f, 1.0f, 1.0f, 1};
int x,z;
float tx,tz;
	
	//glMaterialfv(GL_FRONT, GL_SPECULAR, matspec);
	//glMaterialf(GL_FRONT, GL_SHININESS, 64.0f);

		glPushMatrix();
		glTranslatef(-0.5*scale*TERR_WIDTH, 0, -0.5*scale*TERR_DEPTH);
		glColor3f(1.0f, 1.0f ,1.0f);		//height defined colour

	switch(mode){
	//-------------------------------------------------------------------------------------------------------------------------
	case MODE_FLAT:
		glDisable(GL_TEXTURE_2D);						// Enable Texture Mapping ( NEW )

		for(z=0; z<TERR_DEPTH-2; z++){
			glBegin(GL_TRIANGLE_STRIP);			
			glColor3f(1.0f, 1.0f, 1.0f);		//height defined colour

			for(x=0; x<TERR_WIDTH-1; x++){
		//	glColor3f(normal_elevation.height[x][z+1], 1.0f ,normal_elevation.height[x][z]);		//height defined colour
				
				glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);	
				glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);
				
				//glColor3f(normal_elevation.height[x][z+1], 1.0f ,normal_elevation.height[x][z+1]);
				glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
				glVertex3f(x*scale, elevation.height[x][z+1]*scale, z*scale+(scale));
			}	
			glEnd();
		}
		break;
		glEnable(GL_TEXTURE_2D);						// Enable Texture Mapping ( NEW )

	//-------------------------------------------------------------------------------------------------------------------------
	case MODE_TEXTURE_FULL:

		tx = 1.0f/float(TERR_WIDTH);
		tz = 1.0f/float(TERR_DEPTH);
		for(z=0; z<TERR_DEPTH-2; z++){
			glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){			
				glTexCoord2f(x*tx, z*tz);
				glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);
				glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);			
				
				glTexCoord2f(x*tx, (z+1)*tz);
				glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
				glVertex3f(x*scale, elevation.height[x][z+1]*scale, z*scale+(scale));
			}	
			glEnd();
		}
		break;	
	

	//-------------------------------------------------------------------------------------------------------------------------
	case MODE_TEXTURE_REPEAT:

		tx = 1.0f/float(TERR_WIDTH);
		tz = 1.0f/float(TERR_DEPTH);
		for(z=0; z<TERR_DEPTH-2; z++){
			glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){			
				glTexCoord2f(x, z);
				glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);
				glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);			
				
				glTexCoord2f(x, z+1);
				glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
				glVertex3f(x*scale, elevation.height[x][z+1]*scale, z*scale+(scale));
			}	
			glEnd();
		}
		break;	

	//-------------------------------------------------------------------------------------------------------------------------
	case MODE_TEXTURE_ARB:

		tx = 1.0f/float(TERR_WIDTH);
		tz = 1.0f/float(TERR_DEPTH);
		for(z=0; z<TERR_DEPTH-2; z++){
			glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){	

				glTexCoord2f(x*arb, z*arb);
				glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);
				glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);			
				
				glTexCoord2f(x*arb, (z+1)*arb);
				glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
				glVertex3f(x*scale, elevation.height[x][z+1]*scale, z*scale+(scale));
			}	
			glEnd();
		}
		break;	

	case MODE_TEXTURE_MULTI:
		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texturelist[2]);
		
		//texture unit 1
		glActiveTextureARB(GL_TEXTURE1_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texturelist[3]);
		
		
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
		glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 2);

		tx = 1.0f/float(TERR_WIDTH);
		tz = 1.0f/float(TERR_DEPTH);
		for(z=0; z<TERR_DEPTH-2; z++){
			glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){	
				
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, x*arb, z*arb);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0, normal_elevation.height[x][z]);
				glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);
				glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);			
				

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, x*arb, (z+1)*arb);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0, normal_elevation.height[x][z+1]);

				glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
				glVertex3f(x*scale, elevation.height[x][z+1]*scale, z*scale+(scale));
			}	
			glEnd();
		}
		
		glActiveTextureARB(GL_TEXTURE1_ARB);		
		glDisable(GL_TEXTURE_2D);	
		glActiveTextureARB(GL_TEXTURE0_ARB);		
		glDisable(GL_TEXTURE_2D);	

		//draw sides
			
		glEnable(GL_TEXTURE_2D);	
		glBindTexture(GL_TEXTURE_2D, texturelist[5]);

		//front face
		glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){					
				glTexCoord2f(x*arb, elevation.height[x][TERR_DEPTH-2]*arb);
				glNormal3f(0,0,1);
				glVertex3f(x*scale, elevation.height[x][TERR_DEPTH-2]*scale, (TERR_DEPTH-2)*scale);
				glTexCoord2f(x*arb, 0);
				glVertex3f(x*scale, 0, (TERR_DEPTH-2)*scale);
			}	
			glEnd();

		//back face
		glBegin(GL_TRIANGLE_STRIP);		
			for(x=TERR_WIDTH-2; x>=0; x--){					
				glTexCoord2f(x*arb, elevation.height[x][0]*arb);
				glNormal3f(0,0,-1);
				glVertex3f(x*scale, elevation.height[x][0]*scale, 0);
				glTexCoord2f(x*arb, 0);
				glVertex3f(x*scale, 0, 0);
			}	
			glEnd();

		//left face
		glBegin(GL_TRIANGLE_STRIP);		
			for(z=0; z<TERR_DEPTH-1; z++){					
				glTexCoord2f(elevation.height[0][z]*arb, z*arb);
				glNormal3f(-1,0,0);
				glVertex3f(0, elevation.height[0][z]*scale, z*scale);
				glTexCoord2f(0, z*arb);
				glVertex3f(0, 0, z*scale);
			}	
			glEnd();


		//right face
		glBegin(GL_TRIANGLE_STRIP);		
			for(z=TERR_WIDTH-2; z>=0; z--){						
				glTexCoord2f(elevation.height[TERR_WIDTH-2][z]*arb, z*arb);
				glNormal3f(1,0,0);
				glVertex3f((TERR_WIDTH-2)*scale, elevation.height[TERR_WIDTH-2][z]*scale, z*scale);
				glTexCoord2f(0, z*arb);
				glVertex3f((TERR_WIDTH-2)*scale, 0, z*scale);
			}	
			glEnd();
/*
		//base
				glBegin(GL_QUADS);		
				glVertex3f(0, 0, 0);	
				glVertex3f(TERR_WIDTH, 0, 0);
				glVertex3f(TERR_WIDTH, 0, TERR_DEPTH);
				glVertex3f(0, 0, TERR_DEPTH);
			glEnd();
*/
		break;			
	}





	glPopMatrix();

}
void terrain::DrawNormals (){
int x,z;
	glPushMatrix();
	glTranslatef(-0.5*TERR_WIDTH, 0, -0.5*TERR_DEPTH);

	glColor3f(1.0,1.0,0);

		glBegin(GL_LINES);
	for(z=0; z<=TERR_DEPTH-1; z++){
		for(x=0; x<TERR_WIDTH-1; x++){

			glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);	
			glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);
			glVertex3f(x+normal[x][z].x*scale, elevation.height[x][z]+normal[x][z].y*scale, z+normal[x][z].z*scale);	

		}	
	}
		glEnd();
	glPopMatrix();


}

void terrain::CalcNormals(){
int x, z;
Vec3F vecX, vecZ, normalvect;

	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){

			if ((x > 0) && (x < (TERR_WIDTH-1))) //not an edge vertex
				vecX.set(2, elevation.height[x+1][z] - elevation.height[x-1][z], 0);
			else
				vecX.set(1, elevation.height[x][z], 0);
			
			if ((z > 0) && (z < (TERR_DEPTH-1))) //not an edge vertex
				vecZ.set(0, elevation.height[x][z+1] - elevation.height[x][z-1], 2);
			else
				vecZ.set(0, elevation.height[x][z], 1);

			normalvect = vecX.CrossProduct(vecZ);
			normalvect.Normalize();
			normalvect = normalvect * -1.0f;//adjust normals, shouldnt be necessary
			normal[x][z] = normalvect;

		}
	}

}

void terrain::Wave(){
int x, z;

	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){
		PlannedElevation.height[x][z] = (0.1f*x)*sin(x+z/(0.5*z))+(0.1f*z)*cos(z+x/(0.9*z));
		}
	}
	CalcNormals();
	thinking = TRUE;
}

void terrain::AddBlob (float r, float h)
{

	
	Vec3F posit;
	int	x, z, xMin, xMax, zMin, zMax;
	float s;

	randomPoint(posit);

	
	xMin= posit.x - r;
	xMax= posit.x + r;
	
	zMin= posit.z - r;
	zMax= posit.z + r;

	xMin= MAX(xMin, 0);
	xMax= MIN(xMax, TERR_WIDTH);
	
	zMin= MAX(zMin, 0);
	zMax= MIN(zMax, TERR_DEPTH);

	
	z= zMax;
	while (z-- > zMin)
	{
		x= xMax;
		while (x-- > xMin)
		{
			s = posit.SeperationXZ(x, z);
			if (s < r)
			{
				PlannedElevation.height[x][z] += h * COS(0.5 * PI * s / r);
			}
		}
	}
	CalcNormals();
	thinking = TRUE;
} // terrAddBlob

void terrain::AddBlobAt (float ax, float az, float r, float h)
{

	
	Vec3F posit;
	int	x, z, xMin, xMax, zMin, zMax;
	float s;

	posit.x = int(ax);
	posit.y=0;
	posit.z = int(az);

	
	xMin= posit.x - r;
	xMax= posit.x + r;
	
	zMin= posit.z - r;
	zMax= posit.z + r;

	xMin= MAX(xMin, 0);
	xMax= MIN(xMax, TERR_WIDTH);
	
	zMin= MAX(zMin, 0);
	zMax= MIN(zMax, TERR_DEPTH);

	
	z= zMax;
	while (z-- > zMin)
	{
		x= xMax;
		while (x-- > xMin)
		{
			s = posit.SeperationXZ(x, z);
			if (s < r)
			{
				PlannedElevation.height[x][z] += h * COS(0.5 * PI * s / r);
			}
		}
	}
	CalcNormals();
	thinking = TRUE;
} // terrAddBlob



void terrain::ScaleHeightMap(float magnitude){

int x, z, dir = 1;
//if(magnitude < 0) dir = -1;
magnitude = abs(magnitude);

	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){
			PlannedElevation.height[x][z] += (PlannedElevation.height[x][z] * magnitude);
		}
	}


}

void terrain::Smooth ()
{
	HeightField	newHeight;
	float sum;
	int x, z;

//HACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACK
		for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){
			newHeight.height[x][z] = 0;
		}
	}
//HACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACK


	// ProcesS interior samples
	z= TERR_DEPTH-1;
	while (z-- > 1)
	{
		x= TERR_WIDTH-1;
		while (x-- > 1)
		{
			sum=	
			PlannedElevation.height[x-1][z-1]	+	PlannedElevation.height[x-1][z]	+	PlannedElevation.height[x-1][z+1] + 
			PlannedElevation.height[x][z-1]	+	PlannedElevation.height[x][z]		+	PlannedElevation.height[x][z+1]   + 
			PlannedElevation.height[x+1][z-1]	+	PlannedElevation.height[x+1][z]	+	PlannedElevation.height[x+1][z+1];

			newHeight.height[x][z]= sum * 1.0 / 9;
		}
	}

	
	// Process left and right edges
	z= TERR_DEPTH-1;
	while (z-- > 1)
	{
		sum=	PlannedElevation.height[0][z-1]	+	PlannedElevation.height[0][z]	+	PlannedElevation.height[0][z+1]   + 
				PlannedElevation.height[1][z-1]	+	PlannedElevation.height[1][z]	+	PlannedElevation.height[1][z+1];

		newHeight.height[0][z]= sum * 1.0 / 6;

		sum=	PlannedElevation.height[TERR_WIDTH-1][z-1]	+	PlannedElevation.height[TERR_WIDTH-1][z]	+	PlannedElevation.height[TERR_WIDTH-1][z+1]   + 
				PlannedElevation.height[TERR_WIDTH-2][z-1]	+	PlannedElevation.height[TERR_WIDTH-2][z]	+	PlannedElevation.height[TERR_WIDTH-2][z+1];

		newHeight.height[TERR_WIDTH-1][z]= sum * 1.0 / 6;
	}

	
	// Process far and near edges
	x= TERR_WIDTH-1;
	while (x-- > 1)
	{
		sum=	PlannedElevation.height[x-1][0]	+	PlannedElevation.height[x][0]	+	PlannedElevation.height[x+1][0]   + 
				PlannedElevation.height[x-1][1]	+	PlannedElevation.height[x][1]	+	PlannedElevation.height[x+1][1];

		newHeight.height[x][0]= sum * 1.0 / 6;

		sum=	PlannedElevation.height[x-1][TERR_DEPTH-1]	+	PlannedElevation.height[x][TERR_DEPTH-1]	+	PlannedElevation.height[x+1][TERR_DEPTH-1]   +
				PlannedElevation.height[x-1][TERR_DEPTH-2]	+	PlannedElevation.height[x][TERR_DEPTH-2]	+	PlannedElevation.height[x+1][TERR_DEPTH-2];

		newHeight.height[x][TERR_DEPTH-1]= sum * 1.0 / 6;
	}

	// Process corners
	sum=	PlannedElevation.height[0][0]	+	PlannedElevation.height[1][0] +
			PlannedElevation.height[0][1]	+	PlannedElevation.height[1][1];

	newHeight.height[0][0]= sum * 1.0 / 4;

	sum=	PlannedElevation.height[TERR_WIDTH-1][0]	+	PlannedElevation.height[TERR_WIDTH-2][0] +
			PlannedElevation.height[TERR_WIDTH-1][1]	+	PlannedElevation.height[TERR_WIDTH-2][1];

	newHeight.height[TERR_WIDTH-1][0]= sum * 1.0 / 4;

	sum=	PlannedElevation.height[TERR_WIDTH-1][TERR_DEPTH-1]	+	PlannedElevation.height[TERR_WIDTH-2][TERR_DEPTH-1] +
			PlannedElevation.height[TERR_WIDTH-1][TERR_DEPTH-2]	+	PlannedElevation.height[TERR_WIDTH-2][TERR_DEPTH-2];

	newHeight.height[TERR_WIDTH-1][TERR_DEPTH-1]= sum * 1.0 / 4;

	sum=	PlannedElevation.height[0][TERR_DEPTH-1]	+	PlannedElevation.height[0][TERR_DEPTH-1] +
			PlannedElevation.height[1][TERR_DEPTH-2]	+	PlannedElevation.height[1][TERR_DEPTH-2];

	newHeight.height[0][TERR_DEPTH-1]= sum * 1.0 / 4;


	// Store back to original
	
	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){
			PlannedElevation.height[x][z] = newHeight.height[x][z];
		}
	}


	CalcNormals();
	thinking = TRUE;
} // terrSmooth


void terrain::ApplyFaultPlane (plane &theplane, float dh)
{
	Vec3F vect;
	int x, z;

	z= TERR_DEPTH;
	while (z-- > 0)
	{
		x= TERR_WIDTH;
		while (x-- > 0)
		{
			vect.set(x, elevation.height[x][z], z);

			if (theplane.pointSeperation(vect) > 0)
			{
				PlannedElevation.height[x][z]+= dh;
			}
			else
			{
				if((PlannedElevation.height[x][z]-= dh) < 0)
				{
					PlannedElevation.height[x][z] = 0;
				}

			}
		}
	}
} // terrApplyFaultPlane

float plane::pointSeperation (Vec3F &thispoint)
{
	return( thispoint.DotProduct(norm) + s);

} // return( vecDot(&(pPL->n), pV) + pPL->s);


void terrain::GenFaults (float dh, uint n)
{
	plane faultPlane;
	Vec3F vect;

	while (n-- > 0)
	{
		randomPoint(vect);
		faultPlane.randomise(vect);
		ApplyFaultPlane(faultPlane, dh);
	}
	CalcNormals();
	thinking = TRUE;
} // terrGenFaults

float terrain::normaliseHeight(float theight, bool h){

int x, z, max,min, diff;

max = min = elevation.height[0][0];
	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){
			if(	PlannedElevation.height[x][z] > max) max = PlannedElevation.height[x][z];
			if(	PlannedElevation.height[x][z] < min) min = PlannedElevation.height[x][z];
		}
	}
	diff = max-min;

	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){
			normal_elevation.height[x][z] = ((PlannedElevation.height[x][z]-min)/diff);

			if(h!=0)
			{
				PlannedElevation.height[x][z]	 = normal_elevation.height[x][z] * theight;	
			}
				
		}
	}

	if(h!=0)		
	{
		CalcNormals();		
		thinking = TRUE;
	}

return diff;
}



void terrain::makeFromHeightMap(byte *hmap, float scale, bool add){
int x, z, hX, hZ;
	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){//TERR_WIDTH
		//	printf(" %d",hmap[z+x]);

			hX = x % TERR_WIDTH;					// Error Check Our x Value
			hZ = z % TERR_DEPTH;					// Error Check Our y Value
			if (add !=1) 
				PlannedElevation.height[x][z] = float(hmap[hX + (hZ * TERR_DEPTH)]) * scale;
			else
				PlannedElevation.height[x][z] += float(hmap[hX + (hZ * TERR_DEPTH)]) * scale;
		
		}
	}

	CalcNormals();
	thinking = TRUE;
}

void terrain::makeFromHeightMapI(int hmap[][128], float scale, bool add){
int x, z;
	for(z=0; z<TERR_DEPTH; z++){
		for(x=0; x<TERR_WIDTH; x++){//TERR_WIDTH
			if (add !=1) 
				PlannedElevation.height[x][z] = float(hmap[x][z]) * scale;
			else
				PlannedElevation.height[x][z] += float(hmap[x][z]) * scale;
		
		
		}
	}

	CalcNormals();
	thinking = TRUE;
}

float terrain::heightAtPoint(float tx, float ty){
int ix, iy;
float dx, dy;
float m, n, o;

	tx = (0.5f*TERR_WIDTH) + (tx/scale);
	ty = (0.5f*TERR_DEPTH) + (ty/scale);

	ix = int(tx);
	iy = int(ty);
	dx = tx-ix;
	dy = ty-iy;

	m = elevation.height[ix][iy] + (( elevation.height[ix+1][iy] - elevation.height[ix][iy] ) * dx);
	n = elevation.height[ix][iy+1] + (( elevation.height[ix+1][iy+1] - elevation.height[ix][iy+1] ) * dx);
	o = m + (( n-m ) * (dy));

//printf("\nint (%d, %d)\tdec (%2.2f, %2.2f)\t iheight:  %2.2f, Hx+1:  %2.2f\n m:%2.2f,  n:%2.2f,  o:%2.2f", 
//	   ix, iy, dx, dy, float(elevation.height[ix][iy]),  float(elevation.height[ix+1][iy]), m, n, o);
	//printf("\n---------------------------------------------"); 
return o*scale;
}


Vec3F terrain::DownHillVectorForPoint(float tx, float ty){
float hi_p;
int ax, ay, ix, iy;
Vec3F temp;
temp.x =temp.y =temp.z = 0.0f;

	tx = (0.5f*TERR_WIDTH) + (tx/scale);
	ty = (0.5f*TERR_DEPTH) + (ty/scale);
	ix = int(tx);
	iy = int(ty);

	//Get hightest corner of tile
	hi_p = elevation.height[ix][iy];	ax = 0;	ay = 0;	
	if(elevation.height[ix+1][iy] > hi_p){		hi_p = elevation.height[ix][iy];	ax = 1;	ay = 0;	}
	if(elevation.height[ix][iy+1] > hi_p){		hi_p = elevation.height[ix][iy];	ax = 0;	ay = 1;	}
	if(elevation.height[ix+1][iy+1] > hi_p){	hi_p = elevation.height[ix][iy];	ax = 1;	ay = 1;	}

	if (ax = 0)
		temp.x = hi_p-elevation.height[ix][iy];
	else
		temp.x = hi_p-elevation.height[ix+1][iy];

	if (ay = 0)
		temp.z = hi_p - elevation.height[ix][iy];
	else
		temp.z = hi_p-elevation.height[ix][iy+1];

	temp.y = hi_p - elevation.height[ix+(1-ax)][iy+(1-ay)];

/*
printf("\n-----------------------------------");
printf("\nhigh: %2.2f", hi_p);
printf("\ndown (%2.2f, %2.2f, %2.2f)", temp.x, temp.y, temp.z);
*/
return temp;
}

int terrain::isOnTerrain(float x, float z)
{

return	((x > -(TERR_WIDTH*scale*0.5))	&& 
		(x < TERR_WIDTH*scale*0.5)		&&
		(z > -(TERR_DEPTH*scale*0.5))	&& 
		(z < TERR_DEPTH*scale*0.5)
		);


}



void plane::randomise (Vec3F &vect)
{
	

	// Generate a random direction in the X-Z plane
	norm.set(randF32()-0.5, 0, randF32()-0.5);

	// use as direction for plane normal 
	norm.Normalize();

	// Generate a random position in the X-Z plane
	//thisterrain.randomTerrainPoint(vect);

	// use to generate plane separation from origin
	s= -norm.DotProduct(vect);
	
} // terrRandPlane


void terrain::GenDisplayList ()
{
static float matspec[]= {1.0f, 1.0f, 1.0f, 1};
int x,z;
float tx,tz;
displayListTerr = glGenLists(1);
glNewList(displayListTerr,GL_COMPILE);

		glPushMatrix();
		glTranslatef(-0.5*scale*TERR_WIDTH, 0, -0.5*scale*TERR_DEPTH);
		glColor3f(1.0f, 1.0f ,1.0f);		//height defined colour

		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texturelist[2]);
		
		//texture unit 1
		glActiveTextureARB(GL_TEXTURE1_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texturelist[3]);
		
		
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
		glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 2);

		tx = 1.0f/float(TERR_WIDTH);
		tz = 1.0f/float(TERR_DEPTH);
		for(z=0; z<TERR_DEPTH-2; z++){
			glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){	
				
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, x*texscale, z*texscale);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0, normal_elevation.height[x][z]);
				glNormal3f(normal[x][z].x, normal[x][z].y, normal[x][z].z);
				glVertex3f(x*scale, elevation.height[x][z]*scale, z*scale);			
				

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, x*texscale, (z+1)*texscale);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0, normal_elevation.height[x][z+1]);

				glNormal3f(normal[x][z+1].x, normal[x][z+1].y, normal[x][z+1].z);	
				glVertex3f(x*scale, elevation.height[x][z+1]*scale, z*scale+(scale));
			}	
			glEnd();
		}
		
		glActiveTextureARB(GL_TEXTURE1_ARB);		
		glDisable(GL_TEXTURE_2D);	
		glActiveTextureARB(GL_TEXTURE0_ARB);		
		glDisable(GL_TEXTURE_2D);	

		//draw sides
			
		glEnable(GL_TEXTURE_2D);	
		glBindTexture(GL_TEXTURE_2D, texturelist[5]);

		//front face
		glBegin(GL_TRIANGLE_STRIP);		
			for(x=0; x<TERR_WIDTH-1; x++){					
				glTexCoord2f(x*texscale, elevation.height[x][TERR_DEPTH-2]*texscale);
				glNormal3f(0,0,1);
				glVertex3f(x*scale, elevation.height[x][TERR_DEPTH-2]*scale, (TERR_DEPTH-2)*scale);
				glTexCoord2f(x*texscale, 0);
				glVertex3f(x*scale, 0, (TERR_DEPTH-2)*scale);
			}	
			glEnd();

		//back face
		glBegin(GL_TRIANGLE_STRIP);		
			for(x=TERR_WIDTH-2; x>=0; x--){					
				glTexCoord2f(x*texscale, elevation.height[x][0]*texscale);
				glNormal3f(0,0,-1);
				glVertex3f(x*scale, elevation.height[x][0]*scale, 0);
				glTexCoord2f(x*texscale, 0);
				glVertex3f(x*scale, 0, 0);
			}	
			glEnd();

		//left face
		glBegin(GL_TRIANGLE_STRIP);		
			for(z=0; z<TERR_DEPTH-1; z++){					
				glTexCoord2f(elevation.height[0][z]*texscale, z*texscale);
				glNormal3f(-1,0,0);
				glVertex3f(0, elevation.height[0][z]*scale, z*scale);
				glTexCoord2f(0, z*texscale);
				glVertex3f(0, 0, z*scale);
			}	
			glEnd();


		//right face
		glBegin(GL_TRIANGLE_STRIP);		
			for(z=TERR_WIDTH-2; z>=0; z--){						
				glTexCoord2f(elevation.height[TERR_WIDTH-2][z]*texscale, z*texscale);
				glNormal3f(1,0,0);
				glVertex3f((TERR_WIDTH-2)*scale, elevation.height[TERR_WIDTH-2][z]*scale, z*scale);
				glTexCoord2f(0, z*texscale);
				glVertex3f((TERR_WIDTH-2)*scale, 0, z*scale);
			}	
			glEnd();

		glPopMatrix();

glEndList();
}


void terrain::drawDisplayList()
{
	if(	thinking == FALSE )
	{
		glCallList(displayListTerr);
	}
	else
	{
		Draw(MODE_TEXTURE_MULTI, texscale);
	}

}