
/*
	Copyright A N Houston, University of Abertay Dundee, July 2002.

	$Id$
*/

#include "Header/vector.h"



/*** COMPILATION DIRECTIVES ***/


/*** CONSTANTS ***/


/*** TYPES ***/


/*** MACROS ***/


/*** GLOBALS ***/


/*** PRIVATE FUNCTIONS ***/


/*** PUBLIC FUNCTIONS ***/

VectorPtr vecSet (VectorPtr pV, float x, float y, float z)
{
	pV->x= x;
	pV->y= y;
	pV->z= z;
	return(pV);
} // vecSet

VectorPtr vecRand (VectorPtr pV, float x, float y, float z)
{
	pV->x= (randF32()*x*2)-x;
	pV->y= (randF32()*y*2)-y;
	pV->z= (randF32()*z*2)-z;
	return(pV);
} // vecRand

VectorPtr vecPerturb (VectorPtr pV, float x, float y, float z)
{
	Vector v;

	vecRand(&v, x, y, z);
	return vecAdd(pV, pV, &v);
} // vecPerturb



VectorPtr vecRandDir (VectorPtr pV, float l)
{
	float m= 0;

	while (m == 0)
	{
		vecRand(pV, 1, 1, 1);
		m= vecLength(pV);
	}
	return vecScale(pV, pV, l / m);
} // vecRandDir

float vecLength (VectorPtr pV)
{
	return SQRT(SQR(pV->x) + SQR(pV->y) + SQR(pV->z));
} // vecLength

float vecDot (VectorPtr pV1, VectorPtr pV2)
{
	return(pV1->x * pV2->x + pV1->y * pV2->y + pV1->z * pV2->z);
} // vecDot

VectorPtr vecAdd (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2)
{
	pVR->x= pV1->x + pV2->x;
	pVR->y= pV1->y + pV2->y;
	pVR->z= pV1->z + pV2->z;
	return(pVR);
} // vecAdd

VectorPtr vecSub (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2)
{
	pVR->x= pV1->x - pV2->x;
	pVR->y= pV1->y - pV2->y;
	pVR->z= pV1->z - pV2->z;
	return(pVR);
} // vecSub

VectorPtr vecScale (VectorPtr pVR, VectorPtr pV1, float s)
{
	pVR->x= pV1->x * s;
	pVR->y= pV1->y * s;
	pVR->z= pV1->z * s;
	return(pVR);
} // vecScale

VectorPtr vecNorm (VectorPtr pVR, VectorPtr pV1)
{
	vecScale(pVR, pV1, 1.0f / vecLength(pV1) );
	return(pVR);
} // vecNorm

VectorPtr vecCross (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2)
{
	pVR->x= pV1->y * pV2->z - pV1->z * pV2->y;
	pVR->y= pV1->z * pV2->x - pV1->x * pV2->z;
	pVR->z= pV1->x * pV2->y - pV1->y * pV2->x;
	return(pVR);
} // vecCross

VectorPtr vecLerp (VectorPtr pVR, VectorPtr pV1, VectorPtr pV2, float s)
{
	pVR->x= pV1->x + (pV2->x - pV1->x) * s;
	pVR->y= pV1->y + (pV2->y - pV1->y) * s;
	pVR->z= pV1->z + (pV2->z - pV1->z) * s;
	return(pVR);
} // vecLERP

void vecTest (void)
{
	Vector v1={1,0,0};
	Vector v2={0,1,0};
	Vector vr;
	float l, d;

	vecRand(&vr, 1, 1, 1);

	vecPerturb(&vr, 1, 1, 1);

	vecRandDir(&vr, 10);

	l= vecLength(&v1);

	vecScale(&vr, &v1, 1.0f/l);

	l= vecLength(&v2);

	vecNorm(&vr, &v2);

	vecScale(&vr, &vr, l);

	vecAdd(&vr, &v1, &v2);

	vecSub(&vr, &v1, &v2);

	vecCross(&vr, &v2, &v1);

	vecNorm(&v1, &v1);
	vecNorm(&v2, &v2);
	d= vecDot(&v1, &v2);

	vecLerp(&vr, &v1, &v2, 0.5);
} // vectTest
