#include "Header/vectorclass.h"

Vec3F Vec3F::operator+ (Vec3F param) {
  Vec3F temp;
  temp.x = x + param.x;
  temp.y = y + param.y;
  temp.z = z + param.z;
  return (temp);
}

Vec3F Vec3F::operator+= (Vec3F param) {
  Vec3F temp;
  temp.x = x += param.x;
  temp.y = y += param.y;
  temp.z = z += param.z;
  return (temp);
}

Vec3F Vec3F::operator- (Vec3F param) {
  Vec3F temp;
  temp.x = x - param.x;
  temp.y = y - param.y;
  temp.z = z - param.z;
  return (temp);
}

Vec3F Vec3F::operator-= (Vec3F param) {
  Vec3F temp;
  temp.x = x -= param.x;
  temp.y = y -= param.y;
  temp.z = z -= param.z;
  return (temp);
}

Vec3F Vec3F::operator* (Vec3F param) {
  Vec3F temp;
  temp.x = x * param.x;
  temp.y = y * param.y;
  temp.z = z * param.z;
  return (temp);
}

Vec3F Vec3F::operator* (float param) {
  Vec3F temp;
  temp.x = x * param;
  temp.y = y * param;
  temp.z = z * param;
  return (temp);
}


Vec3F Vec3F::operator*= (float param) {
  Vec3F temp;
  temp.x = x *= param;
  temp.y = y *= param;
  temp.z = z *= param;
  return (temp);
}

Vec3F Vec3F::operator/ (Vec3F param) {
  Vec3F temp;
  temp.x = x / param.x;
  temp.y = y / param.y;
  temp.z = z / param.z;
  return (temp);
}

Vec3F Vec3F::operator/ (float param) {
  Vec3F temp;
  temp.x = x / param;
  temp.y = y / param;
  temp.z = z / param;
  return (temp);
}

Vec3F Vec3F::operator/= (float param) {
  Vec3F temp;
  temp.x = x /= param;
  temp.y = y /= param;
  temp.z = z /= param;
  return (temp);
}


Vec3F Vec3F::operator= (float param) {
  Vec3F temp;
  temp.x = param;
  temp.y = param;
  temp.z = param;
  return (temp);
}
//-------------------------------------------------------------------------------


void Vec3F::set (float nx, float ny, float nz)
{
	x = nx;
	y = ny;
	z = nz;
}


void Vec3F::randomise (float nx, float ny, float nz)
{
	x= (randF32()*nx*2)-nx;
	y= (randF32()*ny*2)-ny;
	z= (randF32()*nz*2)-nz;
}


void Vec3F::RandDir (float l){
//sets a vector of unit length pointing in a random direction

	float m= 0;

	while (m == 0)
	{	randomise(1, 1, 1);
		m= Length();
	}

	Scale(l / m);
}


void Vec3F::preturb (float nx, float ny, float nz)
{
	Vec3F temp;
	temp.randomise(nx, ny, nz);
	x += temp.x;
	y += temp.y;
	z += temp.z;
}


float Vec3F::Length(){
// Returns the length of a vector

	return SQRT(SQR(x) + SQR(y) + SQR(z));
}

float Vec3F::LengthNoRoot(){
// Returns the length of a vector

	return (SQR(x) + SQR(y) + SQR(z));
}


void Vec3F::Scale (float s){
//scales the vector by value s
//same as operator *
	x= x * s;
	y= y * s;
	z= z * s;
}

void Vec3F::Normalize(){
// normalizes the vector to 1 unit length;
	Scale(1.0f / Length() );

}

float Vec3F::DotProduct (Vec3F &v2){
//returs the Dot Product of this vector and another one.

	return(x * v2.x + y * v2.y + z * v2.z);
}

Vec3F Vec3F::CrossProduct (Vec3F &v2){
//returns the Cross Product of this vector and another.
	Vec3F temp;

	temp.x= y * v2.z - z * v2.y;
	temp.y= z * v2.x - x * v2.z;
	temp.z= x * v2.y - y * v2.x;
	return(temp);
}


Vec3F Vec3F::Lerp (Vec3F &v2, float s){
//Linearly Interpolates between this vector and another, at value s
	Vec3F temp;

	temp.x= x + (v2.x - x) * s;
	temp.y= y + (v2.y - y) * s;
	temp.z= z + (v2.z - z) * s;
	return(temp);
}

float Vec3F::SeperationXZ (int nx, int nz)
{
	float dx, dz;

	dx=	x - nx;
	dz=	z - nz;

	return SQRT(SQR(dx)+SQR(dz));
} // terrSep

// ##################################### COLOUR 3F ###############################################

Col3F Col3F::operator+ (Col3F param) {
  Col3F temp;
  temp.r = r + param.r;
  temp.g = g + param.g;
  temp.b = b + param.b;
  return (temp);
}


Col3F Col3F::operator/ (float param) {
  Col3F temp;
  temp.r = r / param;
  temp.g = g / param;
  temp.b = b / param;
  return (temp);
}